@@ -35,7 +35,8 @@ from pysollib.app_stat_result import GameStatResult
 from pysollib.app_statistics import Statistics
 from pysollib.cardsetparser import read_cardset_config
 from pysollib.gamedb import GAME_DB, GI, loadGame
-from pysollib.help import destroy_help_html, help_about
+from pysollib.help import (destroy_help_html, help_about, raise_help_html,
+                           unraise_help_html)
 from pysollib.images import Images, SubsampledImages
 from pysollib.mfxutil import Struct, destruct
 from pysollib.mfxutil import USE_PIL
@@ -53,7 +54,14 @@ from pysollib.pysoltk import PysolStatusbar
 from pysollib.pysoltk import SelectCardsetDialogWithPreview
 from pysollib.pysoltk import SelectDialogTreeData
 from pysollib.pysoltk import destroy_find_card_dialog
+from pysollib.pysoltk import destroy_full_picture_dialog
 from pysollib.pysoltk import loadImage, wm_withdraw
+from pysollib.pysoltk import raise_find_card_dialog
+from pysollib.pysoltk import raise_full_picture_dialog
+from pysollib.pysoltk import raise_solver_dialog
+from pysollib.pysoltk import unraise_find_card_dialog
+from pysollib.pysoltk import unraise_full_picture_dialog
+from pysollib.pysoltk import unraise_solver_dialog
 from pysollib.resource import CSI, CardsetManager
 from pysollib.resource import Music, MusicManager
 from pysollib.resource import Sample, SampleManager
@@ -61,6 +69,7 @@ from pysollib.resource import Tile, TileManager
 from pysollib.settings import DEBUG
 from pysollib.settings import PACKAGE, VERSION_TUPLE  # , WIN_SYSTEM
 from pysollib.settings import TOOLKIT
+from pysollib.speech import Speech
 from pysollib.util import IMAGE_EXTENSIONS
 from pysollib.winsystems import TkSettings
 if TOOLKIT == 'tk':
@@ -89,6 +98,7 @@ class Application:
     def __init__(self):
         self.gdb = GAME_DB
         self.opt = Options()
+        self.speech = Speech()
         self.startup_opt = self.opt.copy()
         self.stats = Statistics()
         self.splashscreen = 1
@@ -132,6 +142,7 @@ class Application:
             config=config,
             plugins=os.path.join(config, "plugins"),
             savegames=os.path.join(config, "savegames"),
+            boards=os.path.join(config, "boards"),
             maint=os.path.join(config, "maint"),          # debug
         )
         for k, v in self.dn.__dict__.items():
@@ -276,6 +287,7 @@ class Application:
             #
             destroy_help_html()
             destroy_find_card_dialog()
+            destroy_full_picture_dialog()
             destroy_solver_dialog()
             # update options
             self.opt.last_gameid = id_
@@ -333,6 +345,7 @@ class Application:
                     self.nextgame.loadedgame = tmpgame._loadGame(
                         self.commandline.loadgame, self)
                     self.nextgame.loadedgame.gstats.holded = 0
+                    self.nextgame.id = self.nextgame.loadedgame.id
                 except Exception:
                     traceback.print_exc()
                     self.nextgame.loadedgame = None
@@ -505,9 +518,29 @@ class Application:
             wm_withdraw(self.top)
             self.top.busyUpdate()
 
+    def wm_toggle_fullscreen(self):
+        self.opt.wm_fullscreen = not self.opt.wm_fullscreen
+        self.top.attributes("-fullscreen", self.opt.wm_fullscreen)
+        # Topmost dialogs need to be reset when toggling fullscreen.
+        self.raiseAll()
+        self.top.attributes('-topmost', False)
+
+    def raiseAll(self):
+        raise_find_card_dialog(self.game)
+        raise_full_picture_dialog(self.game)
+        raise_solver_dialog(self.game)
+        raise_help_html(self.game)
+
+    def unraiseAll(self):
+        unraise_find_card_dialog()
+        unraise_full_picture_dialog()
+        unraise_solver_dialog()
+        unraise_help_html()
+
     def loadImages1(self):
         # load dialog images
         dirname = os.path.join("images", "logos")
+        self.gimages.logos = []
         for f in ("joker07_40_774",
                   "joker08_40_774",
                   "joker07_50_774",
@@ -515,23 +548,48 @@ class Application:
                   "joker11_100_774",
                   "joker10_100",):
             self.gimages.logos.append(self.dataloader.findImage(f, dirname))
-        # if WIN_SYSTEM == 'win32':
-        #     dirname = os.path.join('images', 'dialog', 'default')
-        # else:
-        #     dirname = os.path.join('images', 'dialog', 'bluecurve')
-        dirname = os.path.join('images', 'dialog', 'remix')
+        dirname = os.path.join('images', 'dialog', self.opt.dialog_icon_style)
         for f in ('error', 'info', 'question', 'warning'):
             fn = self.dataloader.findImage(f, dirname)
             im = loadImage(fn)
             MfxMessageDialog.img[f] = im
 
         # load button images
-        if 0 and TOOLKIT == 'tk':
-            dirname = os.path.join('images', 'buttons', 'bluecurve')
+        MfxDialog.button_img = {}
+        if TOOLKIT == 'tk' and self.opt.button_icon_style != 'none':
+            dirname = os.path.join('images', 'buttons',
+                                   self.opt.button_icon_style)
             for n, f in (
                 (_('&OK'), 'ok'),
+                (_('&Select'), 'ok'),
+                (_('&Nice'), 'ok'),
+                (_('&Enjoy'), 'ok'),
+                (_("&Great"), 'ok'),
+                (_("&Cool"), 'ok'),
+                (_("&Yeah"), 'ok'),
+                (_("&Wow"), 'ok'),
+                (_("&Oh well"), 'ok'),
+                (_("&That's life"), 'ok'),
+                (_("&Hmm"), 'ok'),
                 (_('&Cancel'), 'cancel'),
+                (_('&Close'), 'cancel'),
+                (_("&Apply"), 'apply'),
+                (_("&Start"), 'apply'),
+                (_('&New'), 'new'),
                 (_('&New game'), 'new'),
+                (_('&Back to game'), 'back'),
+                (_('&Reset...'), 'reset'),
+                (_('&Restart'), 'reset'),
+                (_('&Rules'), 'help'),
+                (_('&Info...'), 'help'),
+                (_('&Credits'), 'help'),
+                (_('&Next number'), 'next'),
+                (_('&Play'), 'next'),
+                (_('&Play this game'), 'next'),
+                (_('C&lear'), 'clear'),
+                (_('&Solid color...'), 'color'),
+                (_('&Save to file'), 'save'),
+                (_('&Statistics...'), 'statistics'),
             ):
                 fn = self.dataloader.findImage(f, dirname)
                 im = loadImage(fn)
@@ -539,16 +597,38 @@ class Application:
 
     def loadImages2(self):
         # load canvas images
-        dirname = "images"
+        dirname = os.path.join("images", "redealicons",
+                               self.opt.redeal_icon_style)
         # for f in ("noredeal", "redeal",):
+        self.gimages.redeal = []
         for f in ("stopsign", "redeal",):
             self.gimages.redeal.append(self.dataloader.findImage(f, dirname))
-        dirname = os.path.join("images", "demo")
-        for f in ("demo01", "demo02", "demo03", "demo04", "demo05",):
-            self.gimages.demo.append(self.dataloader.findImage(f, dirname))
-        dirname = os.path.join("images", "pause")
-        for f in ("pause01", "pause02", "pause03",):
-            self.gimages.pause.append(self.dataloader.findImage(f, dirname))
+        dirname = os.path.join("images", "demo", self.opt.demo_logo_style)
+        self.gimages.demo = []
+        foundall = False
+        count = 0
+        while not foundall:
+            count += 1
+            try:
+                self.gimages.demo.append(self.dataloader.findImage("demo" +
+                                                                   ("%02d" %
+                                                                    (count,)),
+                                                                   dirname))
+            except OSError:
+                foundall = True
+        dirname = os.path.join("images", "pause", self.opt.pause_text_style)
+        self.gimages.pause = []
+        foundall = False
+        count = 0
+        while not foundall:
+            count += 1
+            try:
+                self.gimages.pause.append(self.dataloader.findImage("pause" +
+                                                                    ("%02d" %
+                                                                     (count,)),
+                                                                    dirname))
+            except OSError:
+                foundall = True
         # dirname = os.path.join("images", "stats")
         # for f in ("barchart",):
         #     self.gimages.stats.append(self.dataloader.findImage(f, dirname))
@@ -556,7 +636,7 @@ class Application:
     def loadImages3(self):
         # load treeview images
         SelectDialogTreeData.img = []
-        dirname = os.path.join('images', 'tree')
+        dirname = os.path.join('images', 'tree', self.opt.tree_icon_style)
         for f in ('folder', 'openfolder', 'node', 'emptynode'):
             fn = self.dataloader.findImage(f, dirname)
             im = loadImage(fn)
@@ -616,7 +696,7 @@ class Application:
                 self.opt.colors['table'] = tile.color
                 self.opt.tabletile_name = None
             else:
-                self.opt.tabletile_name = tile.basename
+                self.opt.tabletile_name = tile.name
             self.tabletile_index = i
             self.tabletile_manager.setSelected(i)
             return True
@@ -652,22 +732,25 @@ class Application:
         self.images.setNegative(self.opt.negative_bottom)
         self.subsampled_images.setNegative(self.opt.negative_bottom)
         if update & 1:
-            self.opt.cardset[0] = (cs.name, cs.backname)
+            self.opt.cardset[0][0] = (cs.name, cs.backname)
         if update & 2:
-            self.opt.cardset[cs.si.type] = (cs.name, cs.backname)
+            self.opt.cardset[cs.si.type][cs.si.subtype] = (cs.name,
+                                                           cs.backname)
         gi = self.getGameInfo(id)
         if gi:
             if update & 256:
                 try:
-                    del self.opt.cardset[(1, gi.id)]
+                    del self.opt.cardset[(1, gi.id)][gi.subcategory]
                 except KeyError:
                     pass
             t = self.checkCompatibleCardsetType(gi, cs)
             if not t[1]:
                 if update & 4:
-                    self.opt.cardset[gi.category] = (cs.name, cs.backname)
+                    self.opt.cardset[gi.category][gi.subcategory] = \
+                        (cs.name, cs.backname)
                 if update & 8:
-                    self.opt.cardset[(1, gi.id)] = (cs.name, cs.backname)
+                    self.opt.cardset[(1, gi.id)][gi.subcategory] = \
+                        (cs.name, cs.backname)
         # from pprint import pprint; pprint(self.opt.cardset)
 
     def loadCardset(self, cs, id=0, update=7, progress=None,
@@ -685,14 +768,16 @@ class Application:
         #   key: Cardset.type
         #   value: (Cardset.ident, Images, SubsampledImages)
         c = self.cardsets_cache.get(cs.type)
-        if c and c[0] == cs.ident:
-            # print 'load from cache', c
-            self.images, self.subsampled_images = c[1], c[2]
-            if not tocache:
-                self.updateCardset(id, update=update)
-                if self.menubar is not None:
-                    self.menubar.updateBackgroundImagesMenu()
-            return 1
+        if c:
+            c2 = c.get(cs.subtype)
+            if c2 and c2[0] == cs.ident:
+                # print 'load from cache', c
+                self.images, self.subsampled_images = c2[1], c2[2]
+                if not tocache:
+                    self.updateCardset(id, update=update)
+                    if self.menubar is not None:
+                        self.menubar.updateBackgroundImagesMenu()
+                return 1
         #
         if progress is None and not noprogress:
             self.wm_save_state()
@@ -718,9 +803,13 @@ class Application:
             # if self.opt.save_cardsets:
             c = self.cardsets_cache.get(cs.type)
             if c:
-                # c[1].destruct()
-                destruct(c[1])
-            self.cardsets_cache[cs.type] = (cs.ident, images, simages)
+                c2 = c.get(cs.subtype)
+                if c2:
+                    # c2[1].destruct()
+                    destruct(c2[1])
+            self.cardsets_cache[cs.type] = {}
+            self.cardsets_cache[cs.type][cs.subtype] = (cs.ident, images,
+                                                        simages)
             if not tocache:
                 # elif self.images is not None:
                 #    # self.images.destruct()
@@ -755,9 +844,13 @@ class Application:
 
     def checkCompatibleCardsetType(self, gi, cs):
         assert gi is not None
-        assert cs is not None
+        cs_type = "None"
+        cs_subtype = "None"
         gc = gi.category
-        cs_type = cs.si.type
+        gs = gi.subcategory
+        if cs is not None:
+            cs_type = cs.si.type
+            cs_subtype = cs.si.subtype
         t0, t1 = None, None
         if gc == GI.GC_FRENCH:
             t0 = "French"
@@ -765,6 +858,9 @@ class Application:
                                # CSI.TYPE_TAROCK,
                                ):
                 t1 = t0
+            if (cs_subtype == CSI.SUBTYPE_NONE
+                    and gs == CSI.SUBTYPE_JOKER_DECK):
+                t1 = t0
         elif gc == GI.GC_HANAFUDA:
             t0 = "Hanafuda"
             if cs_type not in (CSI.TYPE_HANAFUDA,):
@@ -806,6 +902,14 @@ class Application:
             t0 = "Matching"
             if cs.ncards < (gi.ncards / 2):    # not enough cards
                 t1 = t0
+        elif gc == GI.GC_PUZZLE:
+            t0 = "Puzzle"
+            if cs_type not in (CSI.TYPE_PUZZLE,) or cs_subtype != gs:
+                t1 = t0
+        elif gc == GI.GC_ISHIDO:
+            t0 = "Ishido"
+            if cs_type not in (CSI.TYPE_ISHIDO,):
+                t1 = t0
         else:
             # we should not come here
             t0 = t1 = "Unknown"
@@ -823,26 +927,29 @@ class Application:
             # try by gameid / category
             for key, flag in (((1, gi.id), 8), (gi.category, 4)):
                 c = self.opt.cardset.get(key)
-                if not c or len(c) != 2:
+                c2 = None
+                if c:
+                    c2 = c.get(gi.subcategory)
+                if not c2 or len(c2) != 2:
                     continue
-                cs = self.cardset_manager.getByName(c[0])
+                cs = self.cardset_manager.getByName(c2[0])
                 if not cs:
                     continue
                 t = self.checkCompatibleCardsetType(gi, cs)
                 if not t[1]:
-                    cs.updateCardback(backname=c[1])
+                    cs.updateCardback(backname=c2[1])
                     return cs, flag, t
         # ask
         return None, 0, t
 
-    def requestCompatibleCardsetType(self, id):
+    def requestCompatibleCardsetType(self, id, progress=None):
         gi = self.getGameInfo(id)
         #
         cs, cs_update_flag, t = self.getCompatibleCardset(gi, self.cardset)
         if cs is self.cardset:
             return 0
         if cs is not None:
-            self.loadCardset(cs, update=1)
+            self.loadCardset(cs, update=1, progress=progress)
             return 1
 
         self.requestCompatibleCardsetTypeDialog(self.cardset, gi, t)
@@ -850,7 +957,7 @@ class Application:
         cs = self.__selectCardsetDialog(t)
         if cs is None:
             return -1
-        self.loadCardset(cs, id=id)
+        self.loadCardset(cs, id=id, progress=progress)
         return 1
 
     def requestCompatibleCardsetTypeDialog(self, cardset, gi, t):
@@ -866,16 +973,24 @@ class Application:
                 'correct_type': t[0]}, strings=(_("&OK"),), default=0)
 
     def selectCardset(self, title, key):
+        wasPaused = False
+        if not self.game.pause:
+            self.game.doPause()
+            wasPaused = True
         d = SelectCardsetDialogWithPreview(
             self.top, title=title, app=self,
             manager=self.cardset_manager, key=key)
+        if self.game.pause:
+            if wasPaused:
+                self.game.doPause()
         cs = self.cardset_manager.get(d.key)
         if d.status != 0 or d.button != 0 or d.key < 0 or cs is None:
             return None
         changed = False
         if USE_PIL:
             if (self.opt.scale_x, self.opt.scale_y,
-                self.opt.auto_scale, self.opt.spread_stacks,
+                self.opt.auto_scale, self.opt.preview_scale,
+                self.opt.spread_stacks,
                 self.opt.preserve_aspect_ratio) != \
                 d.scale_values or \
                     (cs.CARD_XOFFSET, cs.CARD_YOFFSET) != d.cardset_values:
@@ -886,6 +1001,7 @@ class Application:
             (self.opt.scale_x,
              self.opt.scale_y,
              self.opt.auto_scale,
+             self.opt.preview_scale,
              self.opt.spread_stacks,
              self.opt.preserve_aspect_ratio) = d.scale_values
             if not self.opt.auto_scale:
@@ -1189,9 +1305,10 @@ class Application:
             tile = Tile()
             tile.filename = f
             n = image_ext_re.sub("", name)
-            if os.path.split(dirname)[-1] == 'stretch':
+            subdir = os.path.split(dirname)[-1]
+            if subdir == 'stretch' or subdir == 'stretch-4k':
                 tile.stretch = 1
-            if os.path.split(dirname)[-1] == 'save-aspect':
+            if subdir == 'save-aspect' or subdir == 'save-aspect-4k':
                 tile.stretch = 1
                 tile.save_aspect = 1
             # n = re.sub("[-_]", " ", n)
@@ -1209,7 +1326,9 @@ class Application:
             self,
             ("tiles-*",
                 os.path.join("tiles", "stretch"),
-                os.path.join("tiles", "save-aspect")),
+                os.path.join("tiles", "stretch-4k"),
+                os.path.join("tiles", "save-aspect"),
+                os.path.join("tiles", "save-aspect-4k")),
             "PYSOL_TILES")
         # print dirs
         found, t = [], set()
@@ -1232,8 +1351,7 @@ class Application:
             names = list(map(os.path.normcase, names))
             names.sort()
             return names
-        else:
-            return []
+        return []
 
     #
     # init samples / music
