@@ -36,6 +36,7 @@ from pysollib.stack import \
         InitialDealTalonStack, \
         KingAC_RowStack, \
         OpenStack, \
+        RK_FoundationStack, \
         ReserveStack, \
         SS_FoundationStack, \
         SS_RowStack, \
@@ -45,8 +46,9 @@ from pysollib.stack import \
         TalonStack, \
         WasteStack, \
         WasteTalonStack, \
-        Yukon_AC_RowStack
-from pysollib.util import ACE, ANY_SUIT, KING, UNLIMITED_ACCEPTS, \
+        Yukon_AC_RowStack, \
+        Yukon_SS_RowStack
+from pysollib.util import ACE, ANY_SUIT, HEART, KING, UNLIMITED_ACCEPTS, \
         UNLIMITED_MOVES
 
 # ************************************************************************
@@ -61,10 +63,10 @@ class Gypsy(Game):
     RowStack_Class = AC_RowStack
     Hint_Class = KlondikeType_Hint
 
-    def createGame(self, **layout):
+    def createGame(self, rows=8, **layout):
         # create layout
         l, s = Layout(self), self.s
-        kwdefault(layout, rows=8, waste=0, texts=1)
+        kwdefault(layout, rows=rows, waste=0, texts=1)
         self.Layout_Method(l, **layout)
         self.setSize(l.size[0], l.size[1])
         # create stacks
@@ -79,14 +81,28 @@ class Gypsy(Game):
         # default
         l.defaultAll()
 
-    def startGame(self):
+    def startGame(self, flip=0):
         for i in range(2):
-            self.s.talon.dealRow(flip=0, frames=0)
+            self.s.talon.dealRow(flip=flip, frames=0)
         self._startAndDealRow()
 
     shallHighlightMatch = Game._shallHighlightMatch_AC
 
 
+class Gypsy3Decks(Gypsy):
+    def createGame(self, **layout):
+        Gypsy.createGame(self, rows=12)
+
+
+# ************************************************************************
+# * Yeast Dough
+# ************************************************************************
+
+class YeastDough(Gypsy):
+    def startGame(self):
+        Gypsy.startGame(self, flip=1)
+
+
 # ************************************************************************
 # * Giant
 # ************************************************************************
@@ -540,21 +556,28 @@ class Hypotenuse(Gypsy):
     Layout_Method = staticmethod(Layout.klondikeLayout)
     RowStack_Class = KingAC_RowStack
 
-    def createGame(self):
-        Gypsy.createGame(self, rows=10, playcards=24)
+    def createGame(self, rows=10, playcards=24):
+        Gypsy.createGame(self, rows=rows, playcards=playcards)
 
-    def startGame(self, flip=0, reverse=1):
-        for i in range(1, 10):
-            self.s.talon.dealRow(rows=self.s.rows[:i], flip=0, frames=0)
+    def startGame(self, flip=0, rows=10):
+        for i in range(1, rows):
+            self.s.talon.dealRow(rows=self.s.rows[i:], flip=flip, frames=0)
         self._startAndDealRow()
 
 
 class EternalTriangle(Hypotenuse):
 
-    def startGame(self, flip=0, reverse=1):
-        for i in range(1, 10):
-            self.s.talon.dealRow(rows=self.s.rows[i:], frames=0)
-        self._startAndDealRow()
+    def startGame(self):
+        Hypotenuse.startGame(self, flip=1)
+
+
+class SmallTriangle(Hypotenuse):
+
+    def createGame(self):
+        Hypotenuse.createGame(self, rows=7, playcards=20)
+
+    def startGame(self):
+        Hypotenuse.startGame(self, rows=7)
 
 
 class RightTriangle_Talon(OpenStack, DealRowTalonStack):
@@ -986,9 +1009,59 @@ class SwissPatience(Gypsy):
         self._startAndDealRow()
 
 
+# ************************************************************************
+# * Ace of Hearts
+# ************************************************************************
+
+class AceOfHearts_Foundation(RK_FoundationStack):
+    def acceptsCards(self, from_stack, cards):
+        if not self.cards:
+            return len(cards) == 1 and cards[0].suit == HEART and \
+                cards[0].rank == ACE
+        return RK_FoundationStack.acceptsCards(self, from_stack, cards)
+
+    def getBottomImage(self):
+        return self.game.app.images.getSuitBottom(HEART)
+
+
+class AceOfHearts(Game):
+    Hint_Class = YukonType_Hint
+
+    def createGame(self, **layout):
+        # create layout
+        l, s = Layout(self), self.s
+        kwdefault(layout, rows=7, waste=0, texts=1, playcards=30)
+        Layout.klondikeLayout(l, **layout)
+        self.setSize(l.size[0], l.size[1])
+        # create stacks
+        s.talon = DealRowTalonStack(l.s.talon.x, l.s.talon.y, self)
+        if l.s.waste:
+            s.waste = WasteStack(l.s.waste.x, l.s.waste.y, self)
+        r = l.s.foundations[3]
+        s.foundations.append(
+            AceOfHearts_Foundation(r.x, r.y, self, suit=HEART,
+                                   max_cards=52, max_accept=1,
+                                   mod=13))
+        for r in l.s.rows:
+            s.rows.append(Yukon_SS_RowStack(r.x, r.y, self,
+                                            base_rank=KING))
+        # default
+        l.defaultAll()
+
+    def startGame(self):
+        for i in range(1, len(self.s.rows)):
+            self.s.talon.dealRow(
+                rows=self.s.rows[i:], flip=1, frames=0)
+        self.startDealSample()
+        self.s.talon.dealRow()
+
+    shallHighlightMatch = Game._shallHighlightMatch_SS
+
+
 # register the game
 registerGame(GameInfo(1, Gypsy, "Gypsy",
-                      GI.GT_GYPSY, 2, 0, GI.SL_MOSTLY_SKILL))
+                      GI.GT_GYPSY, 2, 0, GI.SL_MOSTLY_SKILL,
+                      altnames=("Double Easthaven", "Normandy",)))
 registerGame(GameInfo(65, Giant, "Giant",
                       GI.GT_GYPSY, 2, 0, GI.SL_MOSTLY_SKILL))
 registerGame(GameInfo(3, Irmgard, "Irmgard",
@@ -996,7 +1069,7 @@ registerGame(GameInfo(3, Irmgard, "Irmgard",
 registerGame(GameInfo(119, DieKoenigsbergerin, "Die Koenigsbergerin",
                       GI.GT_GYPSY, 2, 0, GI.SL_MOSTLY_SKILL))
 registerGame(GameInfo(174, DieRussische, "Russian Patience",
-                      GI.GT_2DECK_TYPE | GI.GT_OPEN | GI.GT_STRIPPED,
+                      GI.GT_KLONDIKE | GI.GT_OPEN | GI.GT_STRIPPED,
                       2, 0, GI.SL_MOSTLY_SKILL,
                       ranks=(0, 6, 7, 8, 9, 10, 11, 12),
                       altnames=("Die Russische",)))
@@ -1012,7 +1085,8 @@ registerGame(GameInfo(217, MilliganHarp, "Milligan Harp",
 registerGame(GameInfo(218, Carlton, "Carlton",
                       GI.GT_GYPSY, 2, 0, GI.SL_MOSTLY_SKILL))
 registerGame(GameInfo(68, LexingtonHarp, "Lexington Harp",
-                      GI.GT_YUKON, 2, 0, GI.SL_BALANCED))
+                      GI.GT_YUKON, 2, 0, GI.SL_BALANCED,
+                      altnames=("Milligan Yukon",)))
 registerGame(GameInfo(154, Brunswick, "Brunswick",
                       GI.GT_YUKON, 2, 0, GI.SL_BALANCED))
 registerGame(GameInfo(121, Mississippi, "Mississippi",
@@ -1023,6 +1097,9 @@ registerGame(GameInfo(226, Blockade, "Blockade",
                       GI.GT_GYPSY, 2, 0, GI.SL_BALANCED))
 registerGame(GameInfo(412, Cone, "Cone",
                       GI.GT_GYPSY, 2, 0, GI.SL_BALANCED))
+registerGame(GameInfo(453, Gypsy3Decks, "Gypsy (3 Decks)",
+                      GI.GT_GYPSY, 3, 0, GI.SL_MOSTLY_SKILL,
+                      altnames=("Triple Easthaven",)))
 registerGame(GameInfo(463, Surprise, "Surprise",
                       GI.GT_GYPSY, 2, 0, GI.SL_BALANCED))
 registerGame(GameInfo(469, PhantomBlockade, "Phantom Blockade",
@@ -1056,13 +1133,21 @@ registerGame(GameInfo(666, TrapdoorSpider, "Trapdoor Spider",
 registerGame(GameInfo(712, Leprechaun, "Leprechaun",
                       GI.GT_GYPSY | GI.GT_ORIGINAL, 2, 0, GI.SL_MOSTLY_SKILL))
 registerGame(GameInfo(718, LockedCards, "Locked Cards",
-                      GI.GT_2DECK_TYPE, 2, 2, GI.SL_BALANCED))
+                      GI.GT_2DECK_TYPE, 2, 2, GI.SL_BALANCED,
+                      altnames=("Prisoners",)))
 registerGame(GameInfo(721, Thirty, "Thirty",
                       GI.GT_1DECK_TYPE | GI.GT_OPEN | GI.GT_STRIPPED, 1, 0,
                       GI.SL_MOSTLY_SKILL, ranks=(0, 6, 7, 8, 9, 10, 11, 12)))
 registerGame(GameInfo(725, TopsyTurvyQueens, "Topsy-Turvy Queens",
                       GI.GT_2DECK_TYPE, 2, 2, GI.SL_BALANCED))
 registerGame(GameInfo(792, KingsSecrets, "King's Secrets",
-                      GI.GT_2DECK_TYPE, 2, 2, GI.SL_BALANCED))
+                      GI.GT_2DECK_TYPE, 2, 2, GI.SL_BALANCED,
+                      altnames=('Royal Secrets', "King's Captives")))
 registerGame(GameInfo(842, SwissPatience, "Swiss Patience",
                       GI.GT_GYPSY, 1, 0, GI.SL_BALANCED))
+registerGame(GameInfo(890, YeastDough, "Yeast Dough",
+                      GI.GT_GYPSY, 2, 0, GI.SL_MOSTLY_SKILL))
+registerGame(GameInfo(912, SmallTriangle, "Small Triangle",
+                      GI.GT_GYPSY, 1, 0, GI.SL_BALANCED))
+registerGame(GameInfo(966, AceOfHearts, "Ace of Hearts",
+                      GI.GT_1DECK_TYPE, 1, 0, GI.SL_MOSTLY_SKILL))
