@@ -1,53 +1,51 @@
-#!/usr/bin/env python2
+#!/usr/bin/env python
 # -*- mode: python; coding: utf-8; -*-
-##---------------------------------------------------------------------------##
-##
-## Copyright (C) 1998-2003 Markus Franz Xaver Johannes Oberhumer
-## Copyright (C) 2003 Mt. Hood Playing Card Co.
-## Copyright (C) 2005-2009 Skomoroh
-##
-## This program is free software: you can redistribute it and/or modify
-## it under the terms of the GNU General Public License as published by
-## the Free Software Foundation, either version 3 of the License, or
-## (at your option) any later version.
-##
-## This program is distributed in the hope that it will be useful,
-## but WITHOUT ANY WARRANTY; without even the implied warranty of
-## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-## GNU General Public License for more details.
-##
-## You should have received a copy of the GNU General Public License
-## along with this program.  If not, see <http://www.gnu.org/licenses/>.
-##
-##---------------------------------------------------------------------------##
+# ---------------------------------------------------------------------------
+#
+# Copyright (C) 1998-2003 Markus Franz Xaver Johannes Oberhumer
+# Copyright (C) 2003 Mt. Hood Playing Card Co.
+# Copyright (C) 2005-2009 Skomoroh
+#
+# This program is free software: you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation, either version 3 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+#
+# ---------------------------------------------------------------------------
 
-__all__ = []
-
-# Imports
-import sys, re
+import re
 import time
-#from tkFont import Font
-from gettext import ungettext
 
-# PySol imports
-from pysollib.gamedb import registerGame, GameInfo, GI
-from pysollib.util import *
-from pysollib.mfxutil import kwdefault, Struct, Image
-from pysollib.stack import *
 from pysollib.game import Game
+from pysollib.gamedb import GI, GameInfo, registerGame
+from pysollib.hint import AbstractHint
 from pysollib.layout import Layout
-from pysollib.hint import AbstractHint, DefaultHint, CautiousDefaultHint
-from pysollib.settings import TOOLKIT, DEBUG
-from pysollib.pysoltk import MfxCanvasText, MfxCanvasImage
-from pysollib.pysoltk import bind, EVENT_HANDLED, ANCHOR_NW
+from pysollib.mfxutil import Image, Struct, kwdefault
+from pysollib.mygettext import _
+from pysollib.mygettext import ungettext
+from pysollib.pysoltk import ANCHOR_NW, EVENT_HANDLED, bind
+from pysollib.pysoltk import MfxCanvasImage, MfxCanvasText
 from pysollib.pysoltk import MfxMessageDialog
+from pysollib.settings import DEBUG, TOOLKIT
+from pysollib.stack import \
+        InitialDealTalonStack, \
+        OpenStack
+from pysollib.util import ANY_SUIT, NO_RANK
 
 
 def factorial(x):
     if x <= 1:
         return 1
     a = 1
-    for i in xrange(x):
+    for i in range(x):
         a *= (i+1)
     return a
 
@@ -71,14 +69,14 @@ class Mahjongg_Hint(AbstractHint):
             for t in stacks[i+1:]:
                 if game.cardsMatch(r.cards[0], t.cards[0]):
                     # simple scoring...
-                    ##score = 10000 + r.id + t.id
+                    # score = 10000 + r.id + t.id
                     rb = r.blockmap
                     tb = t.blockmap
                     score = \
-                          10000 + \
-                          1000 * (len(rb.below) + len(tb.below)) + \
-                          len(rb.all_left) + len(rb.all_right) + \
-                          len(tb.all_left) + len(tb.all_right)
+                        10000 + \
+                        1000 * (len(rb.below) + len(tb.below)) + \
+                        len(rb.all_left) + len(rb.all_right) + \
+                        len(tb.all_left) + len(tb.all_right)
                     self.addHint(score, 1, r, t)
             i += 1
 
@@ -87,7 +85,7 @@ class Mahjongg_Hint(AbstractHint):
 # *
 # ************************************************************************
 
-#class Mahjongg_Foundation(AbstractFoundationStack):
+# class Mahjongg_Foundation(AbstractFoundationStack):
 class Mahjongg_Foundation(OpenStack):
 
     def __init__(self, x, y, game, suit=ANY_SUIT, **cap):
@@ -102,11 +100,11 @@ class Mahjongg_Foundation(OpenStack):
     def basicIsBlocked(self):
         return 1
 
-    #def initBindings(self):
+    # def initBindings(self):
     #    pass
 
     def _position(self, card):
-        #AbstractFoundationStack._position(self, card)
+        # AbstractFoundationStack._position(self, card)
         OpenStack._position(self, card)
 
         fnds = self.game.s.foundations
@@ -117,11 +115,13 @@ class Mahjongg_Foundation(OpenStack):
                 n = i*9+j
                 if fnds[n].cards:
                     fnds[n].group.tkraise()
-        return
 
     def getHelp(self):
         return ''
 
+    def canSelect(self):
+        return False
+
 
 # ************************************************************************
 # *
@@ -165,8 +165,9 @@ class Mahjongg_RowStack(OpenStack):
         self._dropPairMove(ncards, to_stack, frames=-1, shadow=shadow)
 
     def _dropPairMove(self, n, other_stack, frames=-1, shadow=-1):
-        ##print 'drop:', self.id, other_stack.id
-        assert n == 1 and self.acceptsCards(other_stack, [other_stack.cards[-1]])
+        # print 'drop:', self.id, other_stack.id
+        assert n == 1 and self.acceptsCards(
+            other_stack, [other_stack.cards[-1]])
         if not self.game.demo:
             self.game.playSample("droppair", priority=200)
         old_state = self.game.enterState(self.game.S_FILL)
@@ -207,29 +208,61 @@ class Mahjongg_RowStack(OpenStack):
             if rows:
                 self.group.lower(rows[0].group)
                 return
+        elif TOOLKIT == 'kivy':
+            rows = [s for s in self.game.s.rows[:self.id] if s.cards]
+            if rows:
+                self.group.tkraise(rows[-1].group)
+                return
+            rows = [s for s in self.game.s.rows[self.id+1:] if s.cards]
+            if rows:
+                self.group.lower(rows[0].group)
+                return
         elif TOOLKIT == 'gtk':
             # FIXME (this is very slow)
             for s in self.game.s.rows[self.id+1:]:
                 s.group.tkraise()
 
+    def _calcMouseBind(self, binding_format):
+        return self.game.app.opt.calcCustomMouseButtonsBinding(binding_format)
 
     # In Mahjongg games type there are a lot of stacks, so we optimize
     # and don't create bindings that are not used anyway.
     def initBindings(self):
         group = self.group
         # FIXME: dirty hack to access the Stack's private methods
-        #bind(group, "<1>", self._Stack__clickEventHandler)
-        #bind(group, "<3>", self._Stack__controlclickEventHandler)
-        #bind(group, "<Control-1>", self._Stack__controlclickEventHandler)
+        # bind(group, "<1>", self._Stack__clickEventHandler)
+        # bind(group, "<3>", self._Stack__controlclickEventHandler)
+        # bind(group, "<Control-1>", self._Stack__controlclickEventHandler)
         #
-        bind(group, "<1>", self.__clickEventHandler)
-        bind(group, "<3>", self.__controlclickEventHandler)
-        bind(group, "<Control-1>", self.__controlclickEventHandler)
-        ##bind(group, "<Enter>", self._Stack__enterEventHandler)
-        ##bind(group, "<Leave>", self._Stack__leaveEventHandler)
+        bind(
+            group,
+            self._calcMouseBind("<{mouse_button1}>"),
+            self.__clickEventHandler
+        )
+        bind(
+            group,
+            self._calcMouseBind("<{mouse_button3}>"),
+            self.__controlclickEventHandler
+        )
+        bind(
+            group,
+            self._calcMouseBind("<Control-{mouse_button1}>"),
+            self.__controlclickEventHandler
+        )
+        # bind(group, "<Enter>", self._Stack__enterEventHandler)
+        # bind(group, "<Leave>", self._Stack__leaveEventHandler)
+
+    def keyboardAction(self, card, event, type=1):
+        self.keyboard_movement = True
+        self.keyboard_card = card
+        if type == 1:
+            self.__clickEventHandler(event)
+        else:
+            self.__rightclickEventHandler(event)
+        self.keyboard_card = None
 
     def __defaultClickEventHandler(self, event, handler):
-        self.game.event_handled = True # for Game.undoHandler
+        self.game.event_handled = True  # for Game.undoHandler
         if self.game.demo:
             self.game.stopDemo(event)
         if self.game.busy:
@@ -238,7 +271,7 @@ class Mahjongg_RowStack(OpenStack):
         return EVENT_HANDLED
 
     def __clickEventHandler(self, event):
-        ##print 'click:', self.id
+        # print 'click:', self.id
         return self.__defaultClickEventHandler(event, self.clickHandler)
 
     def __controlclickEventHandler(self, event):
@@ -258,8 +291,8 @@ class Mahjongg_RowStack(OpenStack):
             self._stopDrag()
             return 1
         if self.basicIsBlocked():
-            ### remove selection
-            ##self.game.playSample("nomove")
+            # remove selection
+            # self.game.playSample("nomove")
             return 1
         # possible move
         if from_stack:
@@ -267,16 +300,24 @@ class Mahjongg_RowStack(OpenStack):
                 self._stopDrag()
                 # this code actually moves the tiles
                 from_stack.playMoveMove(1, self, frames=0, sound=True)
+                if TOOLKIT == 'kivy':
+                    if drag.shade_img:
+                        # drag.shade_img.dtag(drag.shade_stack.group)
+                        drag.shade_img.delete()
+                        # game.canvas.delete(drag.shade_img)
+                        drag.shade_img = None
                 return 1
         drag.stack = self
         self.game.playSample("startdrag")
+        self.game.app.speech.speak(_("Selected"))
         # create the shade image (see stack.py, _updateShade)
         if drag.shade_img:
-            #drag.shade_img.dtag(drag.shade_stack.group)
+            # drag.shade_img.dtag(drag.shade_stack.group)
             drag.shade_img.delete()
-            #game.canvas.delete(drag.shade_img)
+            # game.canvas.delete(drag.shade_img)
             drag.shade_img = None
-        img = game.app.images.getShadowCard(card.deck, card.suit, card.rank)
+        img = game.app.images.getHighlightedCard(
+            card.deck, card.suit, card.rank)
         if img is None:
             return 1
         img = MfxCanvasImage(game.canvas, self.x, self.y, image=img,
@@ -290,6 +331,7 @@ class Mahjongg_RowStack(OpenStack):
     def cancelDrag(self, event=None):
         if event is None:
             self._stopDrag()
+            self.game.app.speech.speak(_("Unselected"))
 
     def _findCard(self, event):
         # we need to override this because the shade may be hiding
@@ -299,12 +341,23 @@ class Mahjongg_RowStack(OpenStack):
     def getBottomImage(self):
         return None
 
+    def canSelect(self):
+        return len(self.cards) > 0
+
 
 # ************************************************************************
 # *
 # ************************************************************************
 
 class AbstractMahjonggGame(Game):
+    RANKS = ("1", "2", "3", "4", "5", "6", "7", "8", "9", _("Dragon"))
+    SUITS = (_("Strings"), _("Sticks"), _("Coins"))
+    DRAGONS = (_("Green"), _("White"), _("Red"))
+    TRUMPS = (_("North Wind"), _("South Wind"), _("East Wind"),
+              _("West Wind"), _("Winter"), _("Fall"), _("Spring"),
+              _("Summer"), _("Bamboo"), _("Orchid"), _("Plum"),
+              _("Chrysanthemum"))
+
     Hint_Class = Mahjongg_Hint
     RowStack_Class = Mahjongg_RowStack
 
@@ -324,7 +377,7 @@ class AbstractMahjonggGame(Game):
         t = 'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ'
         for i in range(1, len(L), 3):
             n = t.find(L[i])
-            level, height = n / 7, n % 7 + 1
+            level, height = n // 7, n % 7 + 1
             tx = t.find(L[i+1])
             ty = t.find(L[i+2])
             assert n >= 0 and tx >= 0 and ty >= 0
@@ -334,10 +387,206 @@ class AbstractMahjonggGame(Game):
             for tl in range(level, level + height):
                 tiles.append((tl, tx, ty))
         assert len(tiles) == self.NCARDS
-        #tiles.sort()
-        #tiles = tuple(tiles)
+        # tiles.sort()
+        # tiles = tuple(tiles)
         return tiles, max_tl, max_tx, max_ty
 
+    def parseCard(self, card):
+        if not card.face_up:
+            return _("Face-down")
+        if card.suit > 2:
+            return self.TRUMPS[card.rank]
+        if card.rank > 8:
+            return self.DRAGONS[card.suit] + " " + self.RANKS[card.rank]
+        suit = self.SUITS[card.suit]
+        rank = self.RANKS[card.rank]
+        return rank + " - " + suit
+
+    def getStackSpeech(self, stack, cardindex):
+        if stack not in self.s.rows:
+            return Game.getStackSpeech(self, stack, cardindex)
+        if len(stack.cards) == 0:
+            return self.parseEmptyStack(stack)
+        if hasattr(stack, 'blockmap') and len(stack.blockmap.above) > 0:
+            mainCard = _("Covered Tile")
+        else:
+            mainCard = self.parseCard(stack.cards[cardindex])
+        coverCards = ()
+        blockedByCovered = False
+        if (not hasattr(stack, 'blockmap') or
+                not len(stack.blockmap.left) > 0 or
+                not len(stack.blockmap.right) > 0):
+            return mainCard
+        for r in stack.blockmap.above:
+            if r.cards:
+                if len(r.blockmap.above) > 0:
+                    blockedByCovered = True
+                else:
+                    coverCards += (r,)
+        if len(coverCards) > 0 or blockedByCovered:
+            mainCard += " - " + _("Covered by")
+            if blockedByCovered:
+                mainCard += " - " + _("Covered Tiles")
+            for c in coverCards:
+                mainCard += " - " + self.parseCard(c.cards[0])
+        coverCards = ()
+        blockedByCovered = False
+        for r in stack.blockmap.left:
+            if r.cards:
+                if len(r.blockmap.above) > 0:
+                    blockedByCovered = True
+                else:
+                    coverCards += (r,)
+        if len(coverCards) > 0 or blockedByCovered:
+            mainCard += " - " + _("Blocked on left by")
+            if blockedByCovered:
+                mainCard += " - " + _("Covered Tiles")
+            for c in coverCards:
+                mainCard += " - " + self.parseCard(c.cards[0])
+        coverCards = ()
+        blockedByCovered = False
+        for r in stack.blockmap.right:
+            if r.cards:
+                if len(r.blockmap.above) > 0:
+                    blockedByCovered = True
+                else:
+                    coverCards += (r,)
+        if len(coverCards) > 0 or blockedByCovered:
+            mainCard += " - " + _("Blocked on right by")
+            if blockedByCovered:
+                mainCard += " - " + _("Covered Tiles")
+            for c in coverCards:
+                mainCard += " - " + self.parseCard(c.cards[0])
+        return mainCard
+
+    def keyboardSelectLayer(self, direction):
+        oldstack = self.keyboard_selected_stack
+        stack = oldstack
+        if direction == 1:
+            layercheck = oldstack.blockmap.above
+        else:
+            layercheck = oldstack.blockmap.below
+        for checkstack in layercheck:
+            if checkstack.canSelect():
+                stack = checkstack
+        if oldstack != stack:
+            self.keyboard_selected_stack = stack
+            self.keyboard_select_count = 1
+            self._updateKeyboardSelector()
+            if len(stack.cards) > 0:
+                self.app.speech.speak(self.getStackSpeech(stack, -1))
+            else:
+                self.app.speech.speak(self.getStackSpeech(stack, 0))
+        else:
+            self.playSample("edge", priority=200)
+
+    def _getKeyboardSelectStack(self, direction):
+        if self.keyboard_selected_stack is None:
+            for s in self.allstacks:
+                if s.canSelect():
+                    self.keyboard_selected_stack = s
+                    break
+            return
+
+        currentstack = None
+        currentstacky = None
+        currentstackx = None
+        cw, ch = self.app.images.getSize()
+        cs = self.app.images.cs
+
+        scaleh = ch / cs.CARDH
+        scalew = cw / cs.CARDW
+
+        offsetw = cs.SHADOW_XOFFSET * scalew
+        offseth = cs.SHADOW_YOFFSET * scaleh
+
+        for stack in self.allstacks:
+            if (stack in self.s.internals or
+                    stack == self.keyboard_selected_stack or
+                    not stack.canSelect()):
+                continue
+            cs = self.app.images.cs
+            cw2, ch2 = cw, ch
+
+            if (hasattr(self.keyboard_selected_stack, 'blockmap')
+                    and self.keyboard_selected_stack.blockmap.level
+                    != stack.blockmap.level):
+                continue
+
+            if cs.version == 6 or cs.mahjongg3d:
+                cw2 -= offsetw
+                ch2 -= offseth
+            cw2 -= 2
+            ch2 -= 2
+
+            # The x and y are multiplied, to create some distance and prevent
+            # the stacks from overlapping.  There is probably a better
+            # solution.
+            multiplier = 1 + max(scalew, scaleh)
+            sy = stack.y * multiplier
+            sx = stack.x * multiplier
+            ky = self.keyboard_selected_stack.y * multiplier
+            kx = self.keyboard_selected_stack.x * multiplier
+
+            if direction == 0:  # up
+                if ((sy >= ky) or
+                        (sx < kx - cw2 or
+                         sx > kx + cw2) or
+                        (currentstack is not None and
+                         sy < currentstacky)):
+                    continue
+            elif direction == 1:  # down
+                if ((sy <= ky) or
+                        (sx < kx - cw2 or
+                         sx > kx + cw2) or
+                        (currentstack is not None and
+                         sy > currentstacky)):
+                    continue
+            elif direction == 2:  # left
+                if ((sx >= kx) or
+                        (sy < ky - ch2 or
+                         sy > ky + ch2) or
+                        (currentstack is not None and
+                         sx < currentstackx)):
+                    continue
+            elif direction == 3:  # right
+                if ((sx <= kx) or
+                        (sy < ky - ch2 or
+                         sy > ky + ch2) or
+                        (currentstack is not None and
+                         sx > currentstackx)):
+                    continue
+            currentstack = stack
+            currentstackx = sx
+            currentstacky = sy
+        if currentstack is not None:
+            self.keyboard_selected_stack = currentstack
+
+    def keyboardSelectNextType(self, dir=1):
+        oldstack = self.keyboard_selected_stack
+        rows = self.s.rows
+
+        try:
+            index = rows.index(oldstack)
+        except ValueError:
+            if dir > 0:
+                index = -1
+            else:
+                index = 0
+
+        for n in range(1, len(rows) + 1):
+            try_index = (index + (dir * n)) % len(rows)
+            if not rows[try_index].basicIsBlocked():
+                self.keyboard_selected_stack = rows[try_index]
+                break
+
+        stack = self.keyboard_selected_stack
+
+        self._updateKeyboardSelector()
+        if len(stack.cards) > 0:
+            self.app.speech.speak(self.getStackSpeech(stack, -1))
+        else:
+            self.app.speech.speak(self.getStackSpeech(stack, 0))
 
     #
     # game layout
@@ -350,17 +599,23 @@ class AbstractMahjonggGame(Game):
         l, s = Layout(self), self.s
         show_removed = self.app.opt.mahjongg_show_removed
 
-        ##dx, dy = 2, -2
-        ##dx, dy = 3, -3
-        cs = self.app.cardset
-        if cs.version >= 6:
+        # dx, dy = 2, -2
+        # dx, dy = 3, -3
+        cs = self.app.images.cs
+        if cs.version == 6 or cs.mahjongg3d:
             dx = l.XOFFSET
             dy = -l.YOFFSET
             d_x = cs.SHADOW_XOFFSET
             d_y = cs.SHADOW_YOFFSET
-            if self.preview:
+            if self.preview and (not self.app.opt.auto_scale or
+                                 not self.app.opt.preview_scale):
+                size_cap, r = 100, 2
+                if l.CW // r > size_cap or l.CH // r > size_cap:
+                    r = max(l.CW, l.CH) // size_cap
+
                 # Fixme
-                dx, dy, d_x, d_y = dx/2, dy/2, d_x/2, d_y/2
+                dx, dy, d_x, d_y = dx // r, dy // r, d_x // r, d_y // r
+
             self._delta_x, self._delta_y = dx, -dy
         else:
             dx = 3
@@ -368,13 +623,13 @@ class AbstractMahjonggGame(Game):
             d_x = 0
             d_y = 0
             self._delta_x, self._delta_y = 0, 0
-        #print dx, dy, d_x, d_y, cs.version
+        # print dx, dy, d_x, d_y, cs.version
 
         font = self.app.getFont("canvas_default")
 
         # width of self.texts.info
-        #ti_width = Font(self.canvas, font).measure(_('Remaining'))
-        ti_width = 80
+        # ti_width = Font(self.canvas, font).measure(_('Remaining'))
+        ti_width = max(80, l.CW)
 
         # set window size
         dxx, dyy = abs(dx) * (max_tl+1), abs(dy) * (max_tl+1)
@@ -390,10 +645,10 @@ class AbstractMahjonggGame(Game):
             left_margin = l.XM + 4*cardw+fdxx+d_x + l.XM
         else:
             left_margin = l.XM
-        tableau_width = (max_tx+2)*cardw/2+dxx+d_x
+        tableau_width = (max_tx+2)*cardw//2+dxx+d_x
         right_margin = l.XM+ti_width+l.XM
         w = left_margin + tableau_width + right_margin
-        h = l.YM + dyy + (max_ty + 2) * cardh / 2 + d_y + l.YM
+        h = l.YM + dyy + (max_ty + 2) * cardh // 2 + d_y + l.YM
         if show_removed:
             h = max(h, l.YM+fdyy+cardh*9+d_y+l.YM)
         self.setSize(w, h)
@@ -402,21 +657,18 @@ class AbstractMahjonggGame(Game):
         self.check_dist = l.CW*l.CW + l.CH*l.CH     # see _getClosestStack()
 
         # sort tiles (for 3D)
-        tiles.sort(lambda a, b:
-                   cmp(a[0], b[0]) or
-                   cmp(-a[1]+a[2], -b[1]+b[2])
-                   )
+        tiles.sort(key=lambda x: (x[0], x[2]-x[1]))
 
         # create a row stack for each tile and compute the tilemap
         tilemap = {}
         x0 = left_margin
         y0 = l.YM + dyy
         for level, tx, ty in tiles:
-            #print level, tx, ty
-            x = x0 + (tx * cardw) / 2 + level * dx
-            y = y0 + (ty * cardh) / 2 + level * dy
+            # print level, tx, ty
+            x = x0 + (tx * cardw) // 2 + level * dx
+            y = y0 + (ty * cardh) // 2 + level * dy
             stack = self.RowStack_Class(x, y, self)
-            ##stack.G = (level, tx, ty)
+            # stack.G = (level, tx, ty)
             stack.CARD_XOFFSET = dx
             stack.CARD_YOFFSET = dy
             s.rows.append(stack)
@@ -429,7 +681,7 @@ class AbstractMahjonggGame(Game):
         # compute blockmap
         for stack in s.rows:
             level, tx, ty = tiles[stack.id]
-            above, below, left, right, up, bottom = {}, {}, {}, {}, {}, {}
+            above, below, left, right = {}, {}, {}, {}
             # above blockers
             for tl in range(level+1, level+2):
                 above[tilemap.get((tl, tx, ty))] = 1
@@ -449,34 +701,35 @@ class AbstractMahjonggGame(Game):
             right[tilemap.get((level, tx+2, ty))] = 1
             right[tilemap.get((level, tx+2, ty+1))] = 1
             # up blockers
-            ##up[tilemap.get((level, tx, ty-1))] = 1
-            ##up[tilemap.get((level, tx+1, ty-1))] = 1
+            # up[tilemap.get((level, tx, ty-1))] = 1
+            # up[tilemap.get((level, tx+1, ty-1))] = 1
             # bottom blockers
-            ##bottom[tilemap.get((level, tx, ty+2))] = 1
-            ##bottom[tilemap.get((level, tx+1, ty+2))] = 1
+            # bottom[tilemap.get((level, tx, ty+2))] = 1
+            # bottom[tilemap.get((level, tx+1, ty+2))] = 1
             # sanity check - assert that there are no overlapping tiles
             assert tilemap.get((level, tx, ty)) is stack
             assert tilemap.get((level, tx+1, ty)) is stack
             assert tilemap.get((level, tx, ty+1)) is stack
             assert tilemap.get((level, tx+1, ty+1)) is stack
             #
-            above = tuple(filter(None, above.keys()))
-            below = tuple(filter(None, below.keys()))
-            left = tuple(filter(None, left.keys()))
-            right = tuple(filter(None, right.keys()))
-            ##up = tuple(filter(None, up.keys()))
-            ##bottom = tuple(filter(None, bottom.keys()))
+            above = tuple(_f for _f in above.keys() if _f)
+            below = tuple(_f for _f in below.keys() if _f)
+            left = tuple(_f for _f in left.keys() if _f)
+            right = tuple(_f for _f in right.keys() if _f)
+            # up = tuple(filter(None, up.keys()))
+            # bottom = tuple(filter(None, bottom.keys()))
 
             # assemble
             stack.blockmap = Struct(
-                above = above,
-                below = below,
-                left = left,
-                right = right,
-                ##up = up,
-                ##bottom = bottom,
-                all_left = None,
-                all_right = None,
+                above=above,
+                below=below,
+                left=left,
+                right=right,
+                # up=up,
+                # bottom=bottom,
+                all_left=None,
+                all_right=None,
+                level=level
             )
 
         def get_all_left(s):
@@ -487,6 +740,7 @@ class AbstractMahjonggGame(Game):
                     get_all_left(t)
                 s.blockmap.all_left.update(t.blockmap.all_left)
                 s.blockmap.all_left[t] = 1
+
         def get_all_right(s):
             if s.blockmap.all_right is None:
                 s.blockmap.all_right = {}
@@ -503,7 +757,6 @@ class AbstractMahjonggGame(Game):
             r.blockmap.all_left = tuple(r.blockmap.all_left.keys())
             r.blockmap.all_right = tuple(r.blockmap.all_right.keys())
 
-
         # create other stacks
         for i in range(4):
             for j in range(9):
@@ -514,9 +767,12 @@ class AbstractMahjonggGame(Game):
                     if TOOLKIT == 'tk':
                         x = -l.XS-self.canvas.xmargin
                         y = l.YM+dyy
+                    elif TOOLKIT == 'kivy':
+                        x = -1000
+                        y = l.YM+dyy
                     elif TOOLKIT == 'gtk':
                         # FIXME
-                        x = self.width -l.XS
+                        x = self.width - l.XS
                         y = self.height - l.YS
                 stack = Mahjongg_Foundation(x, y, self)
                 if show_removed:
@@ -535,7 +791,6 @@ class AbstractMahjonggGame(Game):
         # Define stack groups
         l.defaultStackGroups()
 
-
     #
     # game overrides
     #
@@ -553,7 +808,6 @@ class AbstractMahjonggGame(Game):
             return cards
         return new_cards
 
-
     def _shuffleHook1(self, cards):
         # old version; it generate a very easy layouts
         old_cards = cards[:]
@@ -583,7 +837,7 @@ class AbstractMahjonggGame(Game):
             c1 = cards[0]
             del cards[0]
             c2 = None
-            for i in xrange(len(cards)):
+            for i in range(len(cards)):
                 if self.cardsMatch(c1, cards[i]):
                     c2 = cards[i]
                     del cards[i]
@@ -596,9 +850,9 @@ class AbstractMahjonggGame(Game):
             if len(free_stacks) < 2:
                 return None             # try another way
             #
-            i = factorial(len(free_stacks))/2/factorial(len(free_stacks)-2)
+            i = factorial(len(free_stacks))//2//factorial(len(free_stacks)-2)
             old_pairs = []
-            for j in xrange(i):
+            for j in range(i):
                 nc = new_cards[:]
                 while True:
                     # create uniq pair
@@ -624,10 +878,9 @@ class AbstractMahjonggGame(Game):
         if new_cards:
             new_cards.reverse()
             return new_cards
-        print 'oops! can\'t create a solvable game'
+        print('oops! can\'t create a solvable game')
         return old_cards
 
-
     def _shuffleHook2(self, rows, cards):
 
         start_time = time.time()
@@ -682,23 +935,24 @@ class AbstractMahjonggGame(Game):
             c1 = cards[0]
             del cards[0]
             c2 = None
-            for i in xrange(len(cards)):
+            for i in range(len(cards)):
                 if self.cardsMatch(c1, cards[i]):
                     c2 = cards[i]
                     del cards[i]
                     break
 
             # find suitable stacks
-##             suitable_stacks = []
-##             for r in rows:
-##                 if nc[r.id] is None and is_suitable(r, nc):
-##                     suitable_stacks.append(r)
+            #  suitable_stacks = []
+            #  for r in rows:
+            #      if nc[r.id] is None and is_suitable(r, nc):
+            #          suitable_stacks.append(r)
             suitable_stacks = [r for r in rows
                                if nc[r.id] is None and is_suitable(r, nc)]
 
             old_pairs = []
-            i = factorial(len(suitable_stacks))/2/factorial(len(suitable_stacks)-2)
-            for j in xrange(i):
+            i = factorial(len(suitable_stacks))//2 \
+                // factorial(len(suitable_stacks)-2)
+            for j in range(i):
                 if iters[0] > max_iters:
                     return None
                 if time.time() - start_time > max_time:
@@ -727,11 +981,11 @@ class AbstractMahjonggGame(Game):
                 if ret:
                     ret = [x for x in ret if x != 1]
                     return ret
-                nc[s1.id] = nc[s2.id] = None # try another way
+                nc[s1.id] = nc[s2.id] = None  # try another way
 
             return None
 
-        new_cards = [None]*len(self.s.rows) # None - empty stack, 1 - non-used
+        new_cards = [None]*len(self.s.rows)  # None - empty stack, 1 - non-used
         drows = dict.fromkeys(rows)     # optimization
         for r in self.s.rows:
             if r not in drows:
@@ -741,16 +995,16 @@ class AbstractMahjonggGame(Game):
         while True:
             ret = create_solvable(cards[:], new_cards)
             if DEBUG:
-                print 'create_solvable time:', time.time() - start_time
+                print('create_solvable time:', time.time() - start_time)
             if ret:
                 ret.reverse()
                 return ret
             if time.time() - start_time > max_time or \
-                   iters[0] <= max_iters:
-                print 'oops! can\'t create a solvable game'
+                    iters[0] <= max_iters:
+                print('oops! can\'t create a solvable game')
                 return None
             iters = [0]
-        print 'oops! can\'t create a solvable game'
+        print('oops! can\'t create a solvable game')
         return None
 
     def _mahjonggShuffle(self):
@@ -784,13 +1038,15 @@ class AbstractMahjonggGame(Game):
 
         new_cards = self._shuffleHook2(rows, cards)
         if new_cards is None:
-            d = MfxMessageDialog(self.top, title=_('Warning'),
+            if TOOLKIT != 'kivy':
+                MfxMessageDialog(self.top, title=_('Warning'),
                                  text=_('''\
 Sorry, I can\'t find
 a solvable configuration.'''),
                                  bitmap='warning')
+
             self.leaveState(old_state)
-            ##self.finishMove()
+            # self.finishMove()
             # hack
             am = self.moves.current[0]
             am.undo(self)               # restore random
@@ -816,7 +1072,7 @@ a solvable configuration.'''),
 
     def startGame(self):
         assert len(self.s.talon.cards) == self.NCARDS
-        #self.s.talon.dealRow(rows = self.s.rows, frames = 0)
+        # self.s.talon.dealRow(rows = self.s.rows, frames = 0)
         n = 12
         self.s.talon.dealRow(rows=self.s.rows[:self.NCARDS-n], frames=0)
         self.startDealSample()
@@ -824,7 +1080,7 @@ a solvable configuration.'''),
         assert len(self.s.talon.cards) == 0
 
     def isGameWon(self):
-        return sum([len(f.cards) for f in self.s.foundations]) == self.NCARDS
+        return sum(len(f.cards) for f in self.s.foundations) == self.NCARDS
 
     def shallHighlightMatch(self, stack1, card1, stack2, card2):
         if stack1.basicIsBlocked() or stack2.basicIsBlocked():
@@ -837,7 +1093,10 @@ a solvable configuration.'''),
     def updateText(self):
         if self.preview > 1 or self.texts.info is None:
             return
+        t = self.getText()
+        self.texts.info.config(text=t)
 
+    def getText(self):
         # find matching tiles
         stacks = []
         for r in self.s.rows:
@@ -849,8 +1108,8 @@ a solvable configuration.'''),
             for t in stacks[i+1:]:
                 if self.cardsMatch(r.cards[0], t.cards[0]):
                     n += 1
-            #if n == 3: n = 1
-            #elif n == 2: n = 0
+            # if n == 3: n = 1
+            # elif n == 2: n = 0
             n = n % 2
             f += n
             i += 1
@@ -861,7 +1120,7 @@ a solvable configuration.'''),
             f = ungettext('%d Free\nMatching\nPair',
                           '%d Free\nMatching\nPairs',
                           f) % f
-        t = sum([len(i.cards) for i in self.s.foundations])
+        t = sum(len(ii.cards) for ii in self.s.foundations)
         r1 = ungettext('%d\nTile\nRemoved\n\n',
                        '%d\nTiles\nRemoved\n\n',
                        t) % t
@@ -870,7 +1129,15 @@ a solvable configuration.'''),
                        self.NCARDS - t) % (self.NCARDS - t)
 
         t = r1 + r2 + f
-        self.texts.info.config(text=t)
+        return t
+
+    def parseStackInfo(self, stack):
+        if not hasattr(stack, 'blockmap'):
+            return ""
+        return _("Layer: %d") % (stack.blockmap.level + 1)
+
+    def parseGameInfo(self):
+        return self.getText()
 
     #
     # Mahjongg special overrides
@@ -880,7 +1147,7 @@ a solvable configuration.'''),
         # Mahjongg special: highlight all moveable tiles
         return ((self.s.rows, 1),)
 
-    def _highlightCards(self, info, sleep=1.5, delta=(1,1,1,1)):
+    def _highlightCards(self, info, sleep=1.5, delta=(1, 1, 1, 1)):
         if not Image:
             delta = (-self._delta_x, 0, 0, -self._delta_y)
             return Game._highlightCards(self, info, sleep=sleep, delta=delta)
@@ -894,9 +1161,9 @@ a solvable configuration.'''),
         for s, c1, c2, color in info:
             assert c1 is c2
             assert c1 in s.cards
-            tkraise = False
             x, y = s.x, s.y
-            img = self.app.images.getHighlightCard(c1.deck, c1.suit, c1.rank)
+            img = self.app.images.getHighlightedCard(
+                c1.deck, c1.suit, c1.rank, 'black')
             if img is None:
                 continue
             img = MfxCanvasImage(self.canvas, x, y, image=img,
@@ -916,9 +1183,8 @@ a solvable configuration.'''),
                 r.delete()
             self.canvas.update_idletasks()
             return EVENT_HANDLED
-        else:
-            # remove items later (find_card_dialog)
-            return items
+        # remove items later (find_card_dialog)
+        return items
 
     def getCardFaceImage(self, deck, suit, rank):
         if suit == 3:
@@ -943,8 +1209,8 @@ a solvable configuration.'''),
         return self.getCardFaceImage(deck, suit, rank)
 
     def _createCard(self, id, deck, suit, rank, x, y):
-        ##if deck >= 1 and suit == 3 and rank >= 4:
-        if deck%4 and suit == 3 and rank >= 4:
+        # if deck >= 1 and suit == 3 and rank >= 4:
+        if deck % 4 and suit == 3 and rank >= 4:
             return None
         return Game._createCard(self, id, deck, suit, rank, x, y)
 
@@ -976,17 +1242,17 @@ a solvable configuration.'''),
         return card1.rank == card2.rank
 
 
-## mahjongg util
+#  mahjongg util
 def comp_cardset(ncards):
     # calc decks, ranks & trumps
     assert ncards % 4 == 0
-    assert 0 < ncards <= 288 # ???
+    assert 0 < ncards <= 288  # ???
     decks = 1
-    cards = ncards/4
+    cards = ncards//4
     if ncards > 144:
         assert ncards % 8 == 0
         decks = 2
-        cards = cards/2
+        cards = cards//2
     ranks, trumps = divmod(cards, 3)
     if ranks > 10:
         trumps += (ranks-10)*3
@@ -1000,25 +1266,24 @@ def comp_cardset(ncards):
 # * register a Mahjongg type game
 # ************************************************************************
 
-from new import classobj
 
 def r(id, short_name, name=None, ncards=144, layout=None):
     assert layout
     if not name:
         name = "Mahjongg " + short_name
-    classname = re.sub('\W', '', name)
+    classname = re.sub('\\W', '', name)
     # create class
-    gameclass = classobj(classname, (AbstractMahjonggGame,), {})
+    gameclass = type(classname, (AbstractMahjonggGame,), {})
     gameclass.L = layout
     gameclass.NCARDS = ncards
     decks, ranks, trumps = comp_cardset(ncards)
     gi = GameInfo(id, gameclass, name,
-                  GI.GT_MAHJONGG, 4*decks, 0, ##GI.SL_MOSTLY_SKILL,
+                  GI.GT_MAHJONGG, 4*decks, 0,  # GI.SL_MOSTLY_SKILL,
                   category=GI.GC_MAHJONGG, short_name=short_name,
-                  suits=range(3), ranks=range(ranks), trumps=range(trumps),
+                  suits=list(range(3)), ranks=list(range(ranks)),
+                  trumps=list(range(trumps)),
                   si={"decks": decks, "ncards": ncards})
     gi.ncards = ncards
     gi.rules_filename = "mahjongg.html"
     registerGame(gi)
     return gi
-
