@@ -38,7 +38,8 @@ from pysollib.stack import \
         SS_FoundationStack, \
         StackWrapper, \
         TalonStack
-from pysollib.util import ANY_RANK, ANY_SUIT, KING, NO_RANK, UNLIMITED_REDEALS
+from pysollib.util import ANY_RANK, ANY_SUIT, JACK, KING, NO_RANK, \
+    QUEEN, UNLIMITED_REDEALS
 
 # ************************************************************************
 # *
@@ -171,10 +172,9 @@ class MonteCarlo(Game):
         diff = stack1.id - stack2.id
         if column == 0:
             return diff in (-5, -4, 1, 5, 6)
-        elif column == 4:
+        if column == 4:
             return diff in (-6, -5, -1, 4, 5)
-        else:
-            return diff in (-6, -5, -4, -1, 1, 4, 5, 6)
+        return diff in (-6, -5, -4, -1, 1, 4, 5, 6)
 
     def fillEmptyStacks(self):
         n = 0
@@ -298,6 +298,9 @@ class Quatorze(MonteCarlo):
         return (stack1.id // 5 == stack2.id // 5 or
                 stack1.id % 5 == stack2.id % 5)
 
+    def shallHighlightMatch(self, stack1, card1, stack2, card2):
+        return card1.rank + card2.rank == 12
+
 
 # ************************************************************************
 # * Simple Pairs
@@ -363,12 +366,82 @@ class BlockTen(SimplePairs):
     def isGameWon(self):
         return len(self.s.foundations[0].cards) == 48
 
+    def shallHighlightMatch(self, stack1, card1, stack2, card2):
+        return (card1.rank + card2.rank == 8 or
+                (9 < card1.rank == card2.rank > 9))
+
 
 class SimpleTens(BlockTen):
     def isGameWon(self):
         return len(self.s.foundations[0].cards) == 36
 
 
+# ************************************************************************
+# * Crispy
+# ************************************************************************
+
+class Crispy_Talon(MonteCarlo_Talon):
+
+    def canDealCards(self):
+        if len(self.cards) == 0:
+            return False
+        return MonteCarlo_Talon.canDealCards(self)
+
+
+class Crispy_RowStack(MonteCarlo_RowStack):
+
+    getBottomImage = BasicRowStack._getReserveBottomImage
+
+    def acceptsCards(self, from_stack, cards):
+        cr = cards[0].rank
+        if len(self.cards) == 0:
+            if cr == KING:
+                return self.id in (1, 2, 13, 14)
+            if cr == QUEEN:
+                return self.id in (4, 7, 8, 11)
+            if cr == JACK:
+                return self.id in (0, 3, 12, 15)
+        if cr in (JACK, QUEEN, KING):
+            return False
+        return MonteCarlo_RowStack.acceptsCards(self, from_stack, cards)
+
+
+class Crispy(SimpleCarlo):
+    Talon_Class = Crispy_Talon
+    RowStack_Class = Crispy_RowStack
+    FILL_STACKS_AFTER_DROP = False
+    FILL_STACKS_BEFORE_SHIFT = True
+
+    def createGame(self):
+        MonteCarlo.createGame(self, rows=4, cols=4)
+
+    def isGameWon(self):
+        for i in (1, 2, 13, 14):
+            if len(self.s.rows[i].cards) != 0 and \
+                    self.s.rows[i].cards[0].rank != KING:
+                return False
+        for i in (4, 7, 8, 11):
+            if len(self.s.rows[i].cards) != 0 and \
+                    self.s.rows[i].cards[0].rank != QUEEN:
+                return False
+        for i in (0, 3, 12, 15):
+            if len(self.s.rows[i].cards) != 0 and \
+                    self.s.rows[i].cards[0].rank != JACK:
+                return False
+        for i in (5, 6, 9, 10):
+            if len(self.s.rows[i].cards) != 0:
+                return False
+        return len(self.s.talon.cards) == 0
+
+    def shiftCards(self):
+        free = 0
+        for r in self.s.rows:
+            assert len(r.cards) <= 1
+            if not r.cards:
+                free += 1
+        return free
+
+
 # ************************************************************************
 # * Neighbour
 # ************************************************************************
@@ -439,6 +512,7 @@ class Neighbour(MonteCarlo):
 # ************************************************************************
 # * Fourteen
 # * Double Fourteen
+# * Juvenile
 # ************************************************************************
 
 class Fourteen_RowStack(MonteCarlo_RowStack):
@@ -475,7 +549,8 @@ class Fourteen(Game):
                                                   dir=0, base_rank=NO_RANK))
         x, y = l.XM + colsperrow * l.XS, l.YM
         s.foundations.append(self.Foundation_Class(x, y, self, suit=ANY_SUIT,
-                             max_move=0, max_cards=52, base_rank=ANY_RANK))
+                             max_move=0, max_cards=(52 * self.gameinfo.decks),
+                                                   base_rank=ANY_RANK))
         l.createText(s.foundations[0], "s")
         x, y = self.width - l.XS, self.height - l.YS
         s.talon = InitialDealTalonStack(x, y, self)
@@ -509,6 +584,17 @@ class DoubleFourteen(Fourteen):
         self.s.talon.dealRow(rows=self.s.rows[:14])
 
 
+class Juvenile(DoubleFourteen):
+
+    def startGame(self):
+        self.startDealSample()
+        for i in range(2):
+            self.s.talon.dealRow(rows=[self.s.rows[0]], frames=0)
+        for i in range(5):
+            self.s.talon.dealRow(rows=self.s.rows[:17], frames=0)
+        self.s.talon.dealRow(rows=self.s.rows[:17])
+
+
 # ************************************************************************
 # * Nestor
 # * Double Nestor
@@ -878,10 +964,9 @@ class DerLetzteMonarch(Game):
         diff = stack1.id - stack2.id
         if column == 0:
             return diff in (-13, 1, 13)
-        elif column == 12:
+        if column == 12:
             return diff in (-13, -1, 13)
-        else:
-            return diff in (-13, -1, 1, 13)
+        return diff in (-13, -1, 1, 13)
 
 
 class TheLastMonarchII(DerLetzteMonarch):
@@ -993,6 +1078,46 @@ class RightAndLeft(Game):
         self._startAndDealRow()
 
 
+# ************************************************************************
+# * Aces Square
+# ************************************************************************
+
+class AcesSquare_RowStack(MonteCarlo_RowStack):
+    def acceptsCards(self, from_stack, cards):
+        if not OpenStack.acceptsCards(self, from_stack, cards):
+            return False
+        if self.cards[-1].rank == 0 or cards[0].rank == 0:
+            return False
+
+        return (self.game.isNeighbour(from_stack, self)
+                and self.cards[-1].suit == cards[0].suit)
+
+
+class AcesSquare(MonteCarlo):
+    Talon_Class = AutoDealTalonStack
+    RowStack_Class = AcesSquare_RowStack
+
+    def createGame(self):
+        MonteCarlo.createGame(self, rows=4, cols=4)
+
+    def isGameWon(self):
+        return len(self.s.foundations[0].cards) == 48
+
+    def fillStack(self, stack):
+        if stack in self.s.rows:
+            if len(stack.cards) == 0 and len(self.s.talon.cards) > 0:
+                self.flipMove(self.s.talon)
+                self.moveMove(1, self.s.talon, stack)
+
+    def isNeighbour(self, stack1, stack2):
+        return (stack1.id // 4 == stack2.id // 4 or
+                stack1.id % 4 == stack2.id % 4)
+
+    def shallHighlightMatch(self, stack1, card1, stack2, card2):
+        return (card1.suit == card2.suit and
+                card1.rank != card2.rank != 0)
+
+
 # register the game
 registerGame(GameInfo(89, MonteCarlo, "Monte Carlo",
                       GI.GT_PAIRING_TYPE, 1, 0, GI.SL_MOSTLY_LUCK,
@@ -1002,14 +1127,16 @@ registerGame(GameInfo(216, MonteCarlo2Decks, "Monte Carlo (2 Decks)",
 registerGame(GameInfo(212, Weddings, "Weddings",
                       GI.GT_PAIRING_TYPE, 1, 0, GI.SL_MOSTLY_LUCK))
 registerGame(GameInfo(90, SimpleCarlo, "Simple Carlo",
-                      GI.GT_PAIRING_TYPE, 1, 0, GI.SL_MOSTLY_LUCK))
+                      GI.GT_PAIRING_TYPE | GI.GT_CHILDREN, 1, 0,
+                      GI.SL_MOSTLY_LUCK))
 registerGame(GameInfo(91, SimplePairs, "Simple Pairs",
-                      GI.GT_PAIRING_TYPE, 1, 0, GI.SL_LUCK,
-                      altnames=("Jamestown", "Pirate Gold", "Treasure Hunt")))
+                      GI.GT_PAIRING_TYPE | GI.GT_CHILDREN, 1, 0,
+                      GI.SL_LUCK, altnames=("Jamestown", "Pirate Gold",
+                                            "Treasure Hunt", "Hunter")))
 registerGame(GameInfo(92, Neighbour, "Neighbour",
                       GI.GT_PAIRING_TYPE, 1, 0, GI.SL_MOSTLY_LUCK))
 registerGame(GameInfo(96, Fourteen, "Fourteen",
-                      GI.GT_PAIRING_TYPE | GI.GT_OPEN, 1, 0,
+                      GI.GT_PAIRING_TYPE | GI.GT_OPEN | GI.GT_CHILDREN, 1, 0,
                       GI.SL_MOSTLY_LUCK, altnames=("Fourteen Out",
                                                    "Fourteen Puzzle",
                                                    "Take Fourteen")))
@@ -1020,12 +1147,13 @@ registerGame(GameInfo(152, DerLetzteMonarch, "The Last Monarch",
                       GI.GT_1DECK_TYPE | GI.GT_OPEN, 1, 0, GI.SL_MOSTLY_SKILL,
                       altnames=("Der letzte Monarch",)))
 registerGame(GameInfo(328, TheWish, "The Wish",
-                      GI.GT_PAIRING_TYPE | GI.GT_STRIPPED, 1, 0,
-                      GI.SL_MOSTLY_LUCK, ranks=(0, 6, 7, 8, 9, 10, 11, 12)))
+                      GI.GT_PAIRING_TYPE | GI.GT_STRIPPED | GI.GT_CHILDREN,
+                      1, 0, GI.SL_MOSTLY_LUCK,
+                      ranks=(0, 6, 7, 8, 9, 10, 11, 12)))
 registerGame(GameInfo(329, TheWishOpen, "The Wish (Open)",
                       GI.GT_PAIRING_TYPE | GI.GT_OPEN | GI.GT_ORIGINAL |
-                      GI.GT_STRIPPED, 1, 0, GI.SL_MOSTLY_SKILL,
-                      ranks=(0, 6, 7, 8, 9, 10, 11, 12),
+                      GI.GT_STRIPPED | GI.GT_CHILDREN, 1, 0,
+                      GI.SL_MOSTLY_SKILL, ranks=(0, 6, 7, 8, 9, 10, 11, 12),
                       rules_filename="thewish.html"))
 registerGame(GameInfo(368, Vertical, "Vertical",
                       GI.GT_PAIRING_TYPE | GI.GT_OPEN, 1, 0,
@@ -1045,8 +1173,8 @@ registerGame(GameInfo(810, Quatorze, "Quatorze",
 registerGame(GameInfo(829, BlockTen, "Block Ten",
                       GI.GT_PAIRING_TYPE, 1, 0, GI.SL_LUCK))
 registerGame(GameInfo(862, SimpleTens, "Simple Tens",
-                      GI.GT_PAIRING_TYPE | GI.GT_STRIPPED, 1, 0, GI.SL_LUCK,
-                      ranks=(0, 1, 2, 3, 4, 5, 6, 7, 8),
+                      GI.GT_PAIRING_TYPE | GI.GT_STRIPPED | GI.GT_CHILDREN,
+                      1, 0, GI.SL_LUCK, ranks=(0, 1, 2, 3, 4, 5, 6, 7, 8),
                       altnames=("Add Up Tens",)))
 registerGame(GameInfo(867, DoubleFourteen, "Double Fourteen",
                       GI.GT_PAIRING_TYPE | GI.GT_OPEN, 2, 0,
@@ -1057,3 +1185,11 @@ registerGame(GameInfo(874, PatientPairs, "Patient Pairs",
 registerGame(GameInfo(875, PatientPairsOpen, "Patient Pairs (Open)",
                       GI.GT_PAIRING_TYPE | GI.GT_OPEN, 1, 0,
                       GI.SL_MOSTLY_SKILL, rules_filename="patientpairs.html"))
+registerGame(GameInfo(898, AcesSquare, "Aces Square",
+                      GI.GT_1DECK_TYPE, 1, 0, GI.SL_BALANCED,
+                      altnames=("Miner",)))
+registerGame(GameInfo(923, Crispy, "Crispy",
+                      GI.GT_1DECK_TYPE, 1, 0, GI.SL_BALANCED))
+registerGame(GameInfo(935, Juvenile, "Juvenile",
+                      GI.GT_PAIRING_TYPE | GI.GT_OPEN, 2, 0,
+                      GI.SL_MOSTLY_LUCK))
