@@ -25,6 +25,7 @@ from pysollib.game import Game
 from pysollib.gamedb import GI, GameInfo, registerGame
 from pysollib.hint import DefaultHint
 from pysollib.layout import Layout
+from pysollib.mygettext import _
 from pysollib.pysoltk import MfxCanvasText
 from pysollib.stack import \
         AbstractFoundationStack, \
@@ -172,6 +173,9 @@ class Pyramid_RowStack(Pyramid_StackMethods, OpenStack):
         OpenStack.copyModel(self, clone)
         clone.blockmap = self.blockmap
 
+    def canSelect(self):
+        return len(self.cards) > 0 and self.cards[-1].face_up
+
 
 # ************************************************************************
 # * Pyramid
@@ -299,6 +303,22 @@ class Pyramid(Game):
     def shallHighlightMatch(self, stack1, card1, stack2, card2):
         return card1.rank + card2.rank == 11
 
+    def getStackSpeech(self, stack, cardindex):
+        if stack not in self.s.rows:
+            return Game.getStackSpeech(self, stack, cardindex)
+        if len(stack.cards) == 0:
+            return self.parseEmptyStack(stack)
+        mainCard = self.parseCard(stack.cards[cardindex])
+        coverCards = ()
+        for r in stack.blockmap:
+            if r.cards:
+                coverCards += (r,)
+        if len(coverCards) > 0:
+            mainCard += " - " + _("Covered by")
+            for c in coverCards:
+                mainCard += " - " + self.parseCard(c.cards[0])
+        return mainCard
+
 
 # ************************************************************************
 # * Relaxed Pyramid
@@ -383,51 +403,18 @@ class PyramidDozen(Giza):
 
 
 # ************************************************************************
-# * Thirteen
-# * FIXME: UNFINISHED
-# * (this doesn't work yet as 2 cards of the Waste should be playable)
+# * Pyramid Thirteen
 # ************************************************************************
 
+# Previous comments suggest there would need to be two waste piles.  This
+# is not true.  Based on AisleRiot's rules, the two waste cards can only
+# be played with each other, which is the same as how PySol's traditional
+# version works.  So the remaining AisleRiot differences are captured
+# in "Pyramid Thirteen", renamed from "Thirteen" to avoid confusion with
+# "Thirteens"
+
 class Thirteen(Pyramid):
-
-    #
-    # game layout
-    #
-
-    def createGame(self):
-        # create layout
-        layout, s = Layout(self), self.s
-
-        # set window
-        self.setSize(7*layout.XS+layout.XM, 5*layout.YS+layout.YM)
-
-        # create stacks
-        for i in range(7):
-            x = layout.XM + (6-i) * layout.XS // 2
-            y = layout.YM + layout.YS + i * layout.YS // 2
-            for j in range(i+1):
-                s.rows.append(Pyramid_RowStack(x, y, self))
-                x = x + layout.XS
-        x, y = layout.XM, layout.YM
-        s.talon = WasteTalonStack(x, y, self, max_rounds=1)
-        layout.createText(s.talon, "s")
-        x = x + layout.XS
-        s.waste = Pyramid_Waste(x, y, self, max_accept=1)
-        layout.createText(s.waste, "s")
-        s.waste.CARD_XOFFSET = 14
-        x, y = self.width - layout.XS, layout.YM
-        s.foundations.append(Pyramid_Foundation(x, y, self,
-                             suit=ANY_SUIT, dir=0, base_rank=ANY_RANK,
-                             max_move=0, max_cards=52))
-
-        # define stack-groups
-        self.sg.talonstacks = [s.talon] + [s.waste]
-        self.sg.openstacks = s.rows + self.sg.talonstacks
-        self.sg.dropstacks = s.rows + self.sg.talonstacks
-
-    #
-    # game overrides
-    #
+    Talon_Class = StackWrapper(Pyramid_Talon, max_rounds=1, max_accept=1)
 
     def startGame(self):
         self.startDealSample()
@@ -441,27 +428,35 @@ class Thirteen(Pyramid):
 # ************************************************************************
 
 class Thirteens(Pyramid):
+    RowStack_Class = Giza_Reserve
+    Foundation_Class = Pyramid_Foundation
+    Talon_Class = AutoDealTalonStack
 
-    def createGame(self):
+    EXTRA_PILE = False
+
+    def createGame(self, rows=2, cols=5):
         # create layout
         layout, s = Layout(self), self.s
 
         # set window
-        self.setSize(layout.XM+5*layout.XS, layout.YM+4*layout.YS)
+        self.setSize(layout.XM + cols * layout.XS,
+                     layout.YM + (rows + 2) * layout.YS)
 
         # create stacks
         x, y = layout.XM, layout.YM
-        for i in range(2):
+        for i in range(rows):
             x = layout.XM
-            for j in range(5):
-                s.rows.append(Giza_Reserve(x, y, self, max_accept=1))
+            for j in range(cols):
+                s.rows.append(self.RowStack_Class(x, y, self, max_accept=1))
                 x += layout.XS
             y += layout.YS
         x, y = layout.XM, self.height-layout.YS
-        s.talon = AutoDealTalonStack(x, y, self)
+        s.talon = self.Talon_Class(x, y, self)
+        if self.EXTRA_PILE:
+            s.reserves.append(self.RowStack_Class(x + layout.XS, y, self))
         layout.createText(s.talon, 'n')
         x, y = self.width-layout.XS, self.height-layout.YS
-        s.foundations.append(Pyramid_Foundation(x, y, self,
+        s.foundations.append(self.Foundation_Class(x, y, self,
                              suit=ANY_SUIT, dir=0, base_rank=ANY_RANK,
                              max_move=0, max_cards=52))
         layout.createText(s.foundations[0], 'n')
@@ -480,8 +475,12 @@ class Thirteens(Pyramid):
                 self.s.talon.moveMove(1, stack)
                 self.leaveState(old_state)
 
+    def getStackSpeech(self, stack, cardindex):
+        return Game.getStackSpeech(self, stack, cardindex)
+
 # ************************************************************************
 # * Elevens
+# * Elevens Too
 # * Suit Elevens
 # ************************************************************************
 
@@ -529,8 +528,11 @@ class Elevens(Pyramid):
 
     RowStack_Class = Elevens_RowStack
     Reserve_Class = Elevens_Reserve
+    Foundation_Class = AbstractFoundationStack
+    Talon_Class = AutoDealTalonStack
+    Waste_Class = None
 
-    def createGame(self, rows=3, cols=3, reserves=3, texts=False):
+    def createGame(self, rows=3, cols=3, reserves=3, maxpiles=-1, texts=False):
 
         layout, s = Layout(self), self.s
 
@@ -543,18 +545,27 @@ class Elevens(Pyramid):
             layout.YM + (rows + rp) * layout.YS)
 
         x, y = self.width-layout.XS, layout.YM
-        s.talon = AutoDealTalonStack(x, y, self)
-        layout.createText(s.talon, 's')
+        s.talon = self.Talon_Class(x, y, self)
+        if self.Waste_Class is None:
+            layout.createText(s.talon, 's')
+        else:
+            layout.createText(s.talon, 'nw')
+            y += layout.YS
+            s.waste = self.Waste_Class(x, y, self)
         x, y = self.width-layout.XS, self.height-layout.YS
-        s.foundations.append(AbstractFoundationStack(x, y, self,
+        s.foundations.append(self.Foundation_Class(x, y, self,
                              suit=ANY_SUIT, max_accept=0,
-                             max_move=0, max_cards=52))
+                             max_move=0, max_cards=52 * self.gameinfo.decks))
         layout.createText(s.foundations[0], 'n')
         y = layout.YM
+        piles = 0
         for i in range(rows):
             x = layout.XM
             for j in range(cols):
+                if 0 < maxpiles <= piles:
+                    break
                 s.rows.append(self.RowStack_Class(x, y, self, max_accept=1))
+                piles += 1
                 x += layout.XS
             y += layout.YS
         x, y = layout.XM, self.height-layout.YS
@@ -594,9 +605,12 @@ class Elevens(Pyramid):
                 s.moveMove(1, self.s.foundations[0], frames=4)
         self.leaveState(old_state)
 
+    def getStackSpeech(self, stack, cardindex):
+        return Game.getStackSpeech(self, stack, cardindex)
+
     def shallHighlightMatch(self, stack1, card1, stack2, card2):
-        # FIXME
-        return False
+        return (card1.rank + card2.rank == 9 or
+                (9 < card1.rank != card2.rank > 9))
 
 
 class ElevensToo(Elevens):
@@ -645,6 +659,166 @@ class SuitElevens(Elevens):
     def createGame(self):
         Elevens.createGame(self, rows=3, cols=5)
 
+    def shallHighlightMatch(self, stack1, card1, stack2, card2):
+        return (Elevens.shallHighlightMatch(self, stack1, card1,
+                                            stack2, card2)
+                and card1.suit == card2.suit)
+
+
+# ************************************************************************
+# * Tens
+# * Nines
+# ************************************************************************
+
+class Tens_RowStack(Elevens_RowStack):
+    ACCEPTED_SUM = 8
+
+
+class Tens_Reserve(ReserveStack):
+    ACCEPTED_CARDS = (9, JACK, QUEEN, KING)
+
+    def acceptsCards(self, from_stack, cards):
+        if not ReserveStack.acceptsCards(self, from_stack, cards):
+            return False
+        c = cards[0]
+        if c.rank not in self.ACCEPTED_CARDS:
+            return False
+        for s in self.game.s.reserves:
+            if s.cards and s.cards[0].rank != c.rank:
+                return False
+        return True
+
+
+class Tens(ElevensToo):
+    RowStack_Class = Tens_RowStack
+    Reserve_Class = Tens_Reserve
+
+    def createGame(self):
+        Elevens.createGame(self, rows=2, cols=7, maxpiles=13, reserves=4)
+
+    def shallHighlightMatch(self, stack1, card1, stack2, card2):
+        return (card1.rank + card2.rank == 8 or
+                (8 < card1.rank == card2.rank > 8))
+
+
+class Nines_RowStack(Elevens_RowStack):
+    ACCEPTED_SUM = 7
+
+    def clickHandler(self, event):
+        if not self.cards:
+            return 0
+        c = self.cards[-1]
+        if c.face_up and c.rank == 8 and not self.basicIsBlocked():
+            self.game.playSample("autodrop", priority=20)
+            self.playMoveMove(1, self.game.s.foundations[0], sound=False)
+            return 1
+        return OpenStack.clickHandler(self, event)
+
+
+class Nines_Reserve(Tens_Reserve):
+    def acceptsCards(self, from_stack, cards):
+        if not ReserveStack.acceptsCards(self, from_stack, cards):
+            return False
+        c = cards[0]
+        if c.rank not in self.ACCEPTED_CARDS:
+            return False
+        for s in self.game.s.reserves:
+            if s.cards and s.cards[0].rank == c.rank:
+                return False
+        return True
+
+
+class Nines_Foundation(AbstractFoundationStack):
+    def acceptsCards(self, from_stack, cards):
+        if cards[0].rank == 8:
+            return True
+        # We accept any nine. Pairs will get delivered by _dropPairMove.
+        return AbstractFoundationStack.acceptsCards(self, from_stack, cards)
+
+
+class Nines(Tens):
+    RowStack_Class = Nines_RowStack
+    Reserve_Class = Nines_Reserve
+    Foundation_Class = Nines_Foundation
+
+    def createGame(self):
+        Elevens.createGame(self, rows=3, cols=3, reserves=4)
+
+    def shallHighlightMatch(self, stack1, card1, stack2, card2):
+        return (card1.rank + card2.rank == 7 or
+                (8 < card1.rank != card2.rank > 8))
+
+
+# ************************************************************************
+# * The Lucky Number
+# ************************************************************************
+
+class TheLuckyNumber_Talon(WasteTalonStack):
+
+    def canDealCards(self):
+        for s in self.game.s.reserves:
+            if s.cards:
+                return False
+        return WasteTalonStack.canDealCards(self)
+
+
+class TheLuckyNumber_Waste(WasteStack):
+    def moveMove(self, ncards, to_stack, frames=-1, shadow=-1):
+        if to_stack in self.game.s.rows and len(to_stack.cards) > 0:
+            self._dropPairMove(ncards, to_stack, frames=-1, shadow=shadow)
+        else:
+            self.game.moveMove(ncards, self, to_stack,
+                               frames=frames, shadow=shadow)
+            self.fillStack()
+
+    def _dropPairMove(self, n, other_stack, frames=-1, shadow=-1):
+        if not self.game.demo:
+            self.game.playSample("droppair", priority=200)
+        if not (n == 1 and other_stack.cards):
+            return
+        old_state = self.game.enterState(self.game.S_FILL)
+        f = self.game.s.foundations[0]
+        self.game.moveMove(n, self, f, frames=frames, shadow=shadow)
+        self.game.moveMove(n, other_stack, f, frames=frames, shadow=shadow)
+        self.game.leaveState(old_state)
+        other_stack.fillStack()
+
+
+class TheLuckyNumber(Elevens):
+    Talon_Class = StackWrapper(TheLuckyNumber_Talon, max_rounds=1)
+    Waste_Class = TheLuckyNumber_Waste
+
+    def createGame(self):
+        Elevens.createGame(self, cols=4)
+
+    def startGame(self):
+        for i in range(3):
+            self.s.talon.dealRow(frames=0)
+        self._startAndDealRow()
+        self.s.talon.dealCards()
+
+    def fillStack(self, stack):
+        old_state = self.enterState(self.S_FILL)
+        reserves_ncards = 0
+        for s in self.s.reserves:
+            if s.cards:
+                reserves_ncards += 1
+        if reserves_ncards == 0:
+            for r in self.s.rows:
+                if not r.cards:
+                    if self.s.waste.cards:
+                        self.s.waste.moveMove(1, r)
+                    elif self.s.talon.cards:
+                        self.s.talon.flipMove()
+                        self.s.talon.moveMove(1, r)
+        elif reserves_ncards == len(self.s.reserves):
+            if not self.demo:
+                self.playSample("droppair", priority=200)
+            for s in self.s.reserves:
+                s.moveMove(1, self.s.foundations[0], frames=4)
+            self.fillStack(stack)
+        self.leaveState(old_state)
+
 
 # ************************************************************************
 # * Fifteens
@@ -670,7 +844,7 @@ class Fifteens_Reserve(ReserveStack):
                 if r in ranks:
                     break
             else:
-                n = sum([i+1 for i in ranks])
+                n = sum(i+1 for i in ranks)
                 t = str(n)
         self.texts.misc.config(text=t)
 
@@ -708,11 +882,109 @@ class Fifteens(Elevens):
                 if reserve_ranks == [9, JACK, QUEEN, KING]:
                     self._dropReserve()
             else:
-                reserve_sum = sum([c.rank+1 for c in reserve.cards])
+                reserve_sum = sum(c.rank+1 for c in reserve.cards)
                 if reserve_sum == 15:
                     self._dropReserve()
         self.leaveState(old_state)
 
+    def shallHighlightMatch(self, stack1, card1, stack2, card2):
+        return card1.rank + card2.rank == 13
+
+
+# ************************************************************************
+# * Eighteens
+# ************************************************************************
+
+class Eighteens_RowStack(Elevens_RowStack):
+
+    def acceptsCards(self, from_stack, cards):
+        return False
+
+    def clickHandler(self, event):
+        game = self.game
+        if not self.cards:
+            return False
+        if self.cards[0].rank == 0:
+            game.playSample("autodrop", priority=20)
+            self.playMoveMove(1, game.s.foundations[0], sound=False)
+            self.fillStack()
+            return True
+        if self.game.s.reserves[0].acceptsCards(self, self.cards):
+            return self.playMoveMove(1, self.game.s.reserves[0])
+
+        return False
+
+
+class Eighteens_Reserve(ReserveStack):
+    def acceptsCards(self, from_stack, cards):
+        sum = 0
+        numcards = 0
+        if len(self.cards) > 3:
+            return False
+        for c in self.cards:
+            if ((c.rank > 9 and cards[0].rank > 9)
+                    or (c.rank == cards[0].rank)):
+                return False
+            if cards[0].rank <= 9 and c.rank <= 9:
+                sum += c.rank + 1
+                numcards += 1
+        newsum = sum + cards[0].rank + 1
+        if newsum > 18 or (numcards == 2 and newsum < 18):
+            return False
+        return True
+
+    def updateText(self):
+        if self.game.preview > 1 or self.texts.misc is None:
+            return
+        t = ''
+        if self.cards:
+            t = 0
+            for c in self.cards:
+                if c.rank < JACK:
+                    t += c.rank + 1
+        self.texts.misc.config(text=t)
+
+
+class Eighteens_Foundation(AbstractFoundationStack):
+    def acceptsCards(self, from_stack, cards):
+        if not AbstractFoundationStack.acceptsCards(self, from_stack, cards):
+            return False
+        # We accept any aces.
+        return cards[0].rank == 0
+
+
+class Eighteens(Fifteens):
+    RowStack_Class = Eighteens_RowStack
+    Reserve_Class = StackWrapper(Eighteens_Reserve, max_cards=4)
+    Foundation_Class = StackWrapper(Eighteens_Foundation, max_accept=1)
+
+    def createGame(self):
+        Elevens.createGame(self, rows=3, cols=4, reserves=1, texts=True)
+
+    def fillStack(self, stack=None):
+        old_state = self.enterState(self.S_FILL)
+        reserve = self.s.reserves[0]
+        if len(reserve.cards) == 0:
+            for r in self.s.rows:
+                if not r.cards and self.s.talon.cards:
+                    self.s.talon.flipMove()
+                    self.s.talon.moveMove(1, r)
+        else:
+            facecards = 0
+            reserve_sum = 0
+            for c in reserve.cards:
+                if c.rank < JACK:
+                    reserve_sum += c.rank + 1
+                else:
+                    facecards += 1
+            if (reserve_sum == 18 and facecards == 1
+                    and len(reserve.cards) == 4):
+                self._dropReserve()
+        self.leaveState(old_state)
+
+    def shallHighlightMatch(self, stack1, card1, stack2, card2):
+        return False  # How?
+
 
 # ************************************************************************
 # * Neptune
@@ -733,15 +1005,114 @@ class Neptune_RowStack(Elevens_RowStack):
         self.fillStack()
 
 
-class Neptune(Elevens):
+class Neptune(Thirteens):
     RowStack_Class = Neptune_RowStack
 
     def createGame(self):
-        Elevens.createGame(self, rows=4, cols=2, reserves=0)
+        Thirteens.createGame(self, rows=2, cols=4)
 
     def isGameWon(self):
         return len(self.s.talon.cards) == 0
 
+    def shallHighlightMatch(self, stack1, card1, stack2, card2):
+        return (card1.rank == card2.rank - 1 or
+                card1.rank == card2.rank + 1)
+
+
+# ************************************************************************
+# * Eight Cards
+# ************************************************************************
+
+class EightCards_RowStack(Elevens_RowStack):
+
+    def acceptsCards(self, from_stack, cards):
+        if from_stack is self or not self.cards or len(cards) != 1:
+            return False
+        c = self.cards[-1]
+        return (c.face_up and cards[0].face_up and
+                (cards[0].rank + c.rank == 9))
+
+    def clickHandler(self, event):
+        game = self.game
+        if self.cards and self.cards[0].rank > 9:
+            game.playSample("autodrop", priority=20)
+            self.playMoveMove(1, game.s.foundations[0], sound=False)
+            self.fillStack()
+            return True
+
+        return False
+
+    # second selection to reserves stack: should also move the
+    # pair - not ?
+    def moveMove(self, ncards, to_stack, frames=-1, shadow=-1):
+        if to_stack in self.game.s.rows + self.game.s.reserves:
+            self._dropPairMove(ncards, to_stack, frames=-1, shadow=shadow)
+        else:
+            self.game.moveMove(ncards, self, to_stack,
+                               frames=frames, shadow=shadow)
+            self.fillStack()
+
+
+class EightCards_Foundation(AbstractFoundationStack):
+    def acceptsCards(self, from_stack, cards):
+        if not AbstractFoundationStack.acceptsCards(self, from_stack, cards):
+            return False
+        # We accept any picture cards.
+        return cards[0].rank > 9
+
+
+class EightCards_Talon(AutoDealTalonStack):
+    def canDealCards(self):
+        if len(self.cards) < 1: return False    # noqa E701
+        return self.game.draws > 0 and len(self.game.s.reserves[0].cards) < 1
+
+    def dealCards(self, sound=False):
+        self.game.playSample("dealwaste")
+        self.flipMove()
+        self.moveMove(1, self.game.s.reserves[0])
+        old_state = self.game.enterState(self.game.S_FILL)
+        self.game.saveStateMove(2 | 16)  # for undo
+        self.game.draws -= 1
+        self.game.saveStateMove(1 | 16)  # for redo
+        self.game.leaveState(old_state)
+
+
+class EightCards(Thirteens):
+    RowStack_Class = EightCards_RowStack
+    Foundation_Class = EightCards_Foundation
+    Talon_Class = EightCards_Talon
+
+    EXTRA_PILE = True
+
+    draws = 2
+
+    def createGame(self):
+        Thirteens.createGame(self, rows=2, cols=4)
+
+    def startGame(self):
+        self._startAndDealRow()
+        self.draws = 2
+
+    def _restoreGameHook(self, game):
+        self.draws = game.loadinfo.draws
+
+    def _loadGameHook(self, p):
+        self.loadinfo.addattr(draws=p.load())
+
+    def _saveGameHook(self, p):
+        p.dump(self.draws)
+
+    def setState(self, state):
+        # restore saved vars (from undo/redo)
+        self.draws = state[0]
+
+    def getState(self):
+        # save vars (for undo/redo)
+        return [self.draws]
+
+    def shallHighlightMatch(self, stack1, card1, stack2, card2):
+        return card1.rank + card2.rank == 9
+
 
 # ************************************************************************
 # * Triple Alliance
@@ -978,35 +1349,47 @@ class Apophis_RowStack(Pyramid_RowStack):
 class Apophis(Pharaohs):
     Hint_Class = Apophis_Hint
     RowStack_Class = Apophis_RowStack
+    Waste_Class = Pyramid_Waste
+    Foundation_Class = Pyramid_Foundation
 
     PYRAMID_Y_FACTOR = 2
+    INVERT = False
+    MAX_ROUNDS = 3
 
     def createGame(self):
         # create layout
         layout, s = Layout(self), self.s
 
         # set window
-        w = layout.XM + 9*layout.XS
-        h = layout.YM + 4*layout.YS
+        cols = 9
+        if self.INVERT:
+            cols = 10
+        w = layout.XM + cols * layout.XS
+        h = layout.YM + 4 * layout.YS
         self.setSize(w, h)
 
         # create stacks
         x, y = layout.XM+1.5*layout.XS, layout.YM
-        s.rows = self._createPyramid(layout, x, y, 7)
+        if self.INVERT:
+            s.rows = self._createInvertedPyramid(layout, x, y, 7)
+        else:
+            s.rows = self._createPyramid(layout, x, y, 7)
 
         x, y = layout.XM, layout.YM
-        s.talon = DealReserveRedealTalonStack(x, y, self, max_rounds=3)
+        s.talon = DealReserveRedealTalonStack(x, y, self,
+                                              max_rounds=self.MAX_ROUNDS)
         layout.createText(s.talon, 'se')
-        layout.createRoundText(s.talon, 'ne')
+        if s.talon.max_rounds > 1:
+            layout.createRoundText(s.talon, 'ne')
 
         y += layout.YS
         for i in range(3):
-            stack = Pyramid_Waste(x, y, self, max_accept=1)
+            stack = self.Waste_Class(x, y, self, max_accept=1)
             s.reserves.append(stack)
             layout.createText(stack, 'se')
             y += layout.YS
         x, y = self.width - layout.XS, layout.YM
-        s.foundations.append(Pyramid_Foundation(x, y, self,
+        s.foundations.append(self.Foundation_Class(x, y, self,
                              suit=ANY_SUIT, dir=0, base_rank=ANY_RANK,
                              max_move=0, max_cards=52))
         layout.createText(s.foundations[0], 'nw')
@@ -1022,6 +1405,47 @@ class Apophis(Pharaohs):
     def shallHighlightMatch(self, stack1, card1, stack2, card2):
         return card1.rank + card2.rank == 11
 
+
+# ************************************************************************
+# * Eleven Triangle
+# ************************************************************************
+
+class ElevenTriangle_StackMethods():
+    def acceptsCards(self, from_stack, cards):
+        if self.basicIsBlocked():
+            return False
+        if from_stack is self or not self.cards or len(cards) != 1:
+            return False
+        c = self.cards[-1]
+        return c.face_up and cards[0].face_up and \
+            (cards[0].rank + c.rank == 9 or (cards[0].rank > 9 and
+                                             cards[0].rank == c.rank))
+
+    def _dropKingClickHandler(self, event):
+        return 0
+
+
+class ElevenTriangle_RowStack(ElevenTriangle_StackMethods, Pyramid_RowStack):
+    pass
+
+
+class ElevenTriangle_Waste(ElevenTriangle_StackMethods, Pyramid_Waste):
+    pass
+
+
+class ElevenTriangle(Apophis):
+    RowStack_Class = ElevenTriangle_RowStack
+    Waste_Class = ElevenTriangle_Waste
+    Foundation_Class = PyramidDozen_Foundation
+
+    INVERT = True
+    MAX_ROUNDS = 1
+
+    def shallHighlightMatch(self, stack1, card1, stack2, card2):
+        return (card1.rank + card2.rank == 9 or
+                (9 < card1.rank == card2.rank > 9))
+
+
 # ************************************************************************
 # * Cheops
 # ************************************************************************
@@ -1390,6 +1814,8 @@ class Hurricane_Reserve(Hurricane_StackMethods, OpenStack):
 
 class Hurricane(Pyramid):
     Hint_Class = Hurricane_Hint
+    RowStack_Class = Hurricane_RowStack
+    Reserve_Class = Hurricane_Reserve
 
     def createGame(self):
         # create layout
@@ -1407,7 +1833,7 @@ class Hurricane(Pyramid):
                         (0, 2), (1, 2), (2, 2), (3, 2),
                         ):
             x, y = layout.XM + 1.5*layout.XS + ww*xx, layout.YM + layout.YS*yy
-            stack = Hurricane_Reserve(x, y, self, max_accept=1)
+            stack = self.Reserve_Class(x, y, self, max_accept=1)
             stack.CARD_XOFFSET, stack.CARD_YOFFSET = layout.XOFFSET, 0
             s.reserves.append(stack)
 
@@ -1415,7 +1841,7 @@ class Hurricane(Pyramid):
         x = layout.XM + 1.5*layout.XS + layout.XS+2*layout.XOFFSET + d//2
         y = layout.YM+layout.YS
         for i in range(3):
-            stack = Hurricane_RowStack(x, y, self, max_accept=1)
+            stack = self.RowStack_Class(x, y, self, max_accept=1)
             s.rows.append(stack)
             x += layout.XS
 
@@ -1445,6 +1871,45 @@ class Hurricane(Pyramid):
             self.s.talon.moveMove(1, stack)
             self.leaveState(old_state)
 
+    def shallHighlightMatch(self, stack1, card1, stack2, card2):
+        return card1.rank + card2.rank == 12
+
+
+# ************************************************************************
+# * Ides of March
+# ************************************************************************
+
+class IdesOfMarch_StackMethods(Pyramid_StackMethods):
+
+    def acceptsCards(self, from_stack, cards):
+        if from_stack is self:
+            return False
+        if len(cards) != 1:
+            return False
+        if not self.cards:
+            return False
+        c1 = self.cards[-1]
+        c2 = cards[0]
+        return (c1.face_up and c2.face_up and
+                (c1.rank + c2.rank == 13 or c1.rank + c2.rank == 0))
+
+
+class IdesOfMarch_RowStack(IdesOfMarch_StackMethods, BasicRowStack):
+    pass
+
+
+class IdesOfMarch_Reserve(IdesOfMarch_StackMethods, OpenStack):
+    pass
+
+
+class IdesOfMarch(Hurricane):
+    RowStack_Class = IdesOfMarch_RowStack
+    Reserve_Class = IdesOfMarch_Reserve
+
+    def shallHighlightMatch(self, stack1, card1, stack2, card2):
+        return (card1.rank + card2.rank == 13 or
+                card1.rank + card2.rank == 0)
+
 
 # register the game
 registerGame(GameInfo(38, Pyramid, "Pyramid",
@@ -1452,16 +1917,17 @@ registerGame(GameInfo(38, Pyramid, "Pyramid",
 registerGame(GameInfo(193, RelaxedPyramid, "Relaxed Pyramid",
                       GI.GT_PAIRING_TYPE | GI.GT_RELAXED, 1, 2,
                       GI.SL_MOSTLY_LUCK,
-                      altnames=("Pyramid's Stones",)))
-# registerGame(GameInfo(44, Thirteen, "Thirteen",
-#                       GI.GT_PAIRING_TYPE, 1, 0))
+                      altnames=("Pyramid's Stones", "Pyramid Clear")))
+registerGame(GameInfo(44, Thirteen, "Pyramid Thirteen",
+                      GI.GT_PAIRING_TYPE, 1, 0, GI.SL_MOSTLY_LUCK))
 registerGame(GameInfo(592, Giza, "Giza",
                       GI.GT_PAIRING_TYPE | GI.GT_OPEN, 1, 0, GI.SL_BALANCED))
 registerGame(GameInfo(593, Thirteens, "Thirteens",
                       GI.GT_PAIRING_TYPE, 1, 0, GI.SL_LUCK,
-                      altnames=('Helsinki',)))
+                      altnames=('Helsinki', "Good Thirteen")))
 registerGame(GameInfo(594, Elevens, "Elevens",
-                      GI.GT_PAIRING_TYPE, 1, 0, GI.SL_LUCK))
+                      GI.GT_PAIRING_TYPE, 1, 0, GI.SL_LUCK,
+                      altnames=('Eleven Away',)))
 registerGame(GameInfo(595, ElevensToo, "Elevens Too",
                       GI.GT_PAIRING_TYPE, 1, 0, GI.SL_LUCK))
 registerGame(GameInfo(596, SuitElevens, "Suit Elevens",
@@ -1472,7 +1938,8 @@ registerGame(GameInfo(619, TripleAlliance, "Triple Alliance",
                       GI.GT_PAIRING_TYPE | GI.GT_OPEN, 1, 0,
                       GI.SL_MOSTLY_SKILL, altnames=('Triplets',)))
 registerGame(GameInfo(655, Pharaohs, "Pharaohs",
-                      GI.GT_PAIRING_TYPE | GI.GT_OPEN, 1, 0, GI.SL_BALANCED))
+                      GI.GT_PAIRING_TYPE | GI.GT_OPEN, 1, 0, GI.SL_BALANCED,
+                      altnames=("Three Pharaohs",)))
 registerGame(GameInfo(657, Baroness, "Baroness",
                       GI.GT_PAIRING_TYPE, 1, 0, GI.SL_BALANCED,
                       altnames=('Five Piles',)))
@@ -1493,7 +1960,7 @@ registerGame(GameInfo(699, DoublePyramid, "Double Pyramid",
                       GI.GT_PAIRING_TYPE, 2, 2, GI.SL_MOSTLY_LUCK))
 registerGame(GameInfo(700, Triangle, "Triangle",
                       GI.GT_PAIRING_TYPE, 1, 2, GI.SL_MOSTLY_LUCK,
-                      altnames=('Yield',)))
+                      altnames=('Yield', 'Funnel')))
 registerGame(GameInfo(701, UpAndDown, "Up and Down",
                       GI.GT_PAIRING_TYPE | GI.GT_ORIGINAL, 2, 2,
                       GI.SL_MOSTLY_LUCK))
@@ -1508,4 +1975,23 @@ registerGame(GameInfo(802, TripleAlliance2Decks, "Triple Alliance (2 Decks)",
 registerGame(GameInfo(846, PyramidDozen, "Pyramid Dozen",
                       GI.GT_PAIRING_TYPE | GI.GT_OPEN, 1, 0, GI.SL_BALANCED))
 registerGame(GameInfo(854, Neptune, "Neptune",
-                      GI.GT_PAIRING_TYPE, 1, 0, GI.SL_BALANCED))
+                      GI.GT_PAIRING_TYPE, 1, 0, GI.SL_BALANCED,
+                      altnames=('Mixtures',)))
+registerGame(GameInfo(916, Tens, "Tens",
+                      GI.GT_PAIRING_TYPE, 1, 0, GI.SL_LUCK,
+                      altnames=('Take Ten',)))
+registerGame(GameInfo(929, EightCards, "Eight Cards",
+                      GI.GT_PAIRING_TYPE, 1, 0, GI.SL_LUCK,
+                      altnames=('Acht Karten',)))
+registerGame(GameInfo(937, TheLuckyNumber, "The Lucky Number",
+                      GI.GT_PAIRING_TYPE, 2, 0, GI.SL_LUCK))
+registerGame(GameInfo(950, Eighteens, "Eighteens",
+                      GI.GT_PAIRING_TYPE, 2, 0, GI.SL_MOSTLY_LUCK,
+                      altnames=("Steel Wheels",)))
+registerGame(GameInfo(961, Nines, "Nines",
+                      GI.GT_PAIRING_TYPE, 1, 0, GI.SL_LUCK))
+registerGame(GameInfo(969, ElevenTriangle, "Eleven Triangle",
+                      GI.GT_PAIRING_TYPE, 1, 0, GI.SL_MOSTLY_LUCK))
+registerGame(GameInfo(974, IdesOfMarch, "Ides of March",
+                      GI.GT_PAIRING_TYPE, 1, 0, GI.SL_MOSTLY_LUCK,
+                      altnames=("XV",)))
