@@ -502,6 +502,7 @@ class PyGameAudioClient(AbstractAudioClient):
     def __init__(self):
         AbstractAudioClient.__init__(self)
         import pygame.mixer
+        import pygame.mixer_music
         import pygame.time
         if os.name == 'nt':
             # for py2exe
@@ -510,7 +511,8 @@ class PyGameAudioClient(AbstractAudioClient):
             import pygame.mixer_music
         self.mixer = pygame.mixer
         self.time = pygame.time
-        self.music = self.mixer.music
+        self.mixer_music = pygame.mixer_music
+        self.music = self.mixer_music
         self.audiodev = self.mixer
         self.sound = None
         self.sound_channel = None
@@ -540,8 +542,7 @@ class PyGameAudioClient(AbstractAudioClient):
         if self.sound_channel and self.sound_channel.get_busy():
             if self.sound_priority >= priority:
                 return 0
-            else:
-                self.sound.stop()
+            self.sound.stop()
         vol = self.app.opt.sound_sample_volume/128.0
         try:
             self.sound = self.mixer.Sound(filename)
@@ -561,7 +562,7 @@ class PyGameAudioClient(AbstractAudioClient):
         self.sound_channel = None
 
     def _playMusicLoop(self):
-        # print '_playMusicLoop'
+        # print('_playMusicLoop')
         music_list = self.music_list
         if not music_list:
             return
@@ -611,7 +612,7 @@ class PyGameAudioClient(AbstractAudioClient):
                 self.music = None
         else:
             if not self.music:
-                self.music = self.mixer.music
+                self.music = self.mixer_music
                 th = Thread(target=self._playMusicLoop)
                 th.start()
             else:
