@@ -29,6 +29,9 @@ import re
 from findcarddialog import connect_game_find_card_dialog, \
         destroy_find_card_dialog
 
+from fullpicturedialog import connect_game_full_picture_dialog, \
+        destroy_full_picture_dialog
+
 import gtk
 from gtk import gdk
 
@@ -81,6 +84,10 @@ class PysolMenubarTk:
             connect_game_find_card_dialog(game)
         else:
             destroy_find_card_dialog()
+        if game.canShowFullPicture():
+            connect_game_full_picture_dialog(game)
+        else:
+            destroy_full_picture_dialog()
 
     #
     # create menubar
@@ -252,7 +259,7 @@ class PysolMenubarTk:
             ]
         for label, action, opt_name, update_game in (
             ('A&uto drop',   'optautodrop', 'autodrop',               False),
-            ('Auto &face up',           '', 'autofaceup',             False),
+            ('Auto &face-up',           '', 'autofaceup',             False),
             ('Auto &deal',              '', 'autodeal',               False),
             ('&Quick play',             '', 'quickplay',              False),
             ('Enable &undo',            '', 'undo',                   False),
@@ -480,10 +487,7 @@ class PysolMenubarTk:
             'options.automaticplay.autodrop':
                 '/menubar/options/automaticplay/optautodrop'
             }
-        if path in path_map:
-            path = path_map[path]
-        else:
-            path = '/menubar/'+path.replace('.', '/')
+        path = path_map.get(path, '/menubar/'+path.replace('.', '/'))
         menuitem = self.top.ui_manager.get_widget(path)
         if not menuitem:
             # print 'WARNING: setMenuState: not found:', path
@@ -685,8 +689,11 @@ class PysolMenubarTk:
                 tile = self.app.tabletile_manager.get(0)
                 tile.color = d.key
                 self.app.setTile(0)
-            elif d.key > 0 and d.key != self.app.tabletile_index:
-                self.app.setTile(d.key)
+            elif d.key > 0 and (d.key != self.app.tabletile_index or
+                                d.preview_scaling !=
+                                self.app.opt.tabletile_scale_method):
+                self.app.setTile(d.key, scaling=d.preview_scaling)
+            self.app.opt.tabletile_scale_method = d.preview_scaling
 
     def mSelectCardsetDialog(self, *event):
         if self._cancelDrag(break_pause=False):
@@ -752,10 +759,6 @@ class PysolMenubarTk:
     def updateAll(self, *event):
         self.app.canvas.updateAll()
 
-    def _setCommentMenu(self, v):
-        # FIXME
-        pass
-
     def _setPauseMenu(self, v):
         # FIXME
         pass
