@@ -101,7 +101,6 @@ class _CanvasItem:
 
     def lower(self, positions=None):
         print('lower', self, positions)
-        return  # don't need?
         #  if positions is None:
         #      pass
         #      ##self._item.lower_to_bottom()
@@ -269,8 +268,7 @@ class MfxCanvasText(_CanvasItem):
     def __getitem__(self, key):
         if key == 'text':
             return self._item.get_property('text')
-        else:
-            raise AttributeError(key)
+        raise AttributeError(key)
     cget = __getitem__
 
 
@@ -325,16 +323,15 @@ class MfxCanvas(gnomecanvas.Canvas):
         assert add is None
         # FIXME
         print('TkCanvas bind:', sequence)
-        return
 
     def cget(self, attr):
         if attr == 'cursor':
             # FIXME
             return gdk.LEFT_PTR
             # return self.get_window().get_cursor(v)
-        elif attr == 'width':
+        if attr == 'width':
             return self.get_size()[0]
-        elif attr == 'height':
+        if attr == 'height':
             return self.get_size()[1]
         print('TkCanvas cget:', attr)
         raise AttributeError(attr)
