@@ -22,6 +22,9 @@
 # ---------------------------------------------------------------------------
 
 import os
+import tkinter
+import tkinter.ttk as ttk
+from collections import UserList
 
 from pysollib.gamedb import GI
 from pysollib.mfxutil import KwStruct, Struct, destruct
@@ -31,13 +34,11 @@ from pysollib.resource import CSI
 from pysollib.ui.tktile.selecttree import SelectDialogTreeData
 from pysollib.ui.tktile.tkutil import bind, unbind_destroy
 
-from six.moves import UserList
-from six.moves import tkinter
-from six.moves import tkinter_ttk as ttk
-
 from .selecttree import SelectDialogTreeCanvas
 from .selecttree import SelectDialogTreeLeaf, SelectDialogTreeNode
-from .tkwidget import MfxDialog, MfxScrolledCanvas, PysolCombo
+from .tkwidget import MfxDialog, MfxScrolledCanvas, PysolButton, \
+                      PysolCheckbutton, PysolCombo, PysolEntry, \
+                      PysolNotebook
 
 # ************************************************************************
 # * Nodes
@@ -134,7 +135,7 @@ class SelectGameData(SelectDialogTreeData):
                     select_func, self.all_games_gi)):
                 continue
             gg.append(SelectGameNode(None, name, select_func))
-        if 1 and gg:
+        if gg:
             s_by_compatibility = SelectGameNode(None, _("by Compatibility"),
                                                 tuple(gg))
         #
@@ -147,7 +148,7 @@ class SelectGameData(SelectDialogTreeData):
                 continue
             name = _("New games in v. %(version)s") % {'version': name}
             gg.append(SelectGameNode(None, name, select_func))
-        if 1 and gg:
+        if gg:
             s_by_pysol_version = SelectGameNode(None, _("by PySol version"),
                                                 tuple(gg))
         s_by_inventors, gg = None, []
@@ -158,7 +159,7 @@ class SelectGameData(SelectDialogTreeData):
                     select_func, self.all_games_gi)):
                 continue
             gg.append(SelectGameNode(None, name, select_func))
-        if 1 and gg:
+        if gg:
             s_by_inventors = SelectGameNode(None, _("by Inventors"),
                                             tuple(gg))
         #
@@ -328,6 +329,12 @@ class SelectGameDialog(MfxDialog):
         if button == 0:                 # Ok or double click
             self.gameid = self.tree.selection_key
             self.tree.n_expansions = 1  # save xyview in any case
+        if button == 1:                 # Cancel button
+            # If the user cancels, revert any cardset change from the preview.
+            if self.app.cardset.name != self.cardset.name:
+                self.app.loadCardset(self.cardset,
+                                     id=self.game.gameinfo.category,
+                                     tocache=True, noprogress=True)
         if button == 10:                # Rules
             doc = self.app.getGameRulesFilename(self.tree.selection_key)
             if not doc:
@@ -358,6 +365,7 @@ class SelectGameDialogWithPreview(SelectGameDialog):
         self.gameid = gameid
         self.bookmark = bookmark
         self.criteria = SearchCriteria()
+        self.cardset = self.app.cardset.copy()
         self.random = None
         if self.TreeDataHolder_Class.data is None:
             self.TreeDataHolder_Class.data = self.TreeData_Class(app)
@@ -389,7 +397,7 @@ class SelectGameDialogWithPreview(SelectGameDialog):
         paned_window.add(left_frame)
         paned_window.add(right_frame)
 
-        notebook = ttk.Notebook(left_frame)
+        notebook = PysolNotebook(left_frame)
         notebook.pack(expand=True, fill='both')
         tree_frame = ttk.Frame(notebook)
         notebook.add(tree_frame, text=_('Tree View'))
@@ -406,14 +414,17 @@ class SelectGameDialogWithPreview(SelectGameDialog):
         # Search
         searchbox = ttk.Frame(search_frame)
         searchText = tkinter.StringVar()
-        self.list_searchlabel = tkinter.Label(searchbox, text="Search:",
+        self.list_searchlabel = tkinter.Label(searchbox, text=_("Search:"),
                                               justify='left', anchor='w')
         self.list_searchlabel.pack(side="top", fill='both', ipadx=1)
-        self.list_searchtext = tkinter.Entry(searchbox,
-                                             textvariable=searchText)
+        self.list_searchtext = PysolEntry(searchbox,
+                                          fieldname=_("Search:"),
+                                          textvariable=searchText)
 
-        self.advSearch = tkinter.Button(searchbox, text='...',
-                                        command=self.advancedSearch)
+        self.advSearch = PysolButton(searchbox, text='...',
+                                     prefixtext=_("Advanced Search"),
+                                     command=self.advancedSearch,
+                                     width=2)
         self.advSearch.pack(side="right")
 
         self.list_searchtext.pack(side="top", fill='both',
@@ -431,6 +442,8 @@ class SelectGameDialogWithPreview(SelectGameDialog):
                        fill='both', ipadx=1)
         self.updateSearchList("")
         bind(self.list, '<<ListboxSelect>>', self.selectSearchResult)
+        bind(self.list, '<FocusIn>',
+             lambda e: self.app.speech.speak(_("Games list")))
         bind(self.list, '<FocusOut>',
              lambda e: self.list.selection_clear(0, 'end'))
 
@@ -472,6 +485,8 @@ class SelectGameDialogWithPreview(SelectGameDialog):
         stats_frame.rowconfigure(6, weight=1)
         # Canvas
         self.preview = MfxScrolledCanvas(right_frame)
+        bind(self.preview.parent, '<Configure>',
+             lambda e: self._configureHandler())
         self.preview.setTile(app, app.tabletile_index,
                              app.opt.tabletile_scale_method, force=True)
         self.preview.grid(row=1, column=0, columnspan=3,
@@ -617,21 +632,21 @@ class SelectGameDialogWithPreview(SelectGameDialog):
                     self.criteria.statisticsOptions[self.criteria.statistics]
                 if statoption == 'played' and won + lost == 0:
                     continue
-                elif statoption == 'won' and won == 0:
+                if statoption == 'won' and won == 0:
                     continue
-                elif statoption == 'not won' and (won != 0 or lost == 0):
+                if statoption == 'not won' and (won != 0 or lost == 0):
                     continue
-                elif statoption == 'not played' and won + lost != 0:
+                if statoption == 'not played' and won + lost != 0:
                     continue
 
             if (self.criteria.popular and
                     not (game.si.game_flags & GI.GT_POPULAR)):
                 continue
             if (self.criteria.recent and
-                    not (game.id in self.app.opt.recent_gameid)):
+                    (game.id not in self.app.opt.recent_gameid)):
                 continue
             if (self.criteria.favorite and
-                    not (game.id in self.app.opt.favorite_gameid)):
+                    (game.id not in self.app.opt.favorite_gameid)):
                 continue
             if (self.criteria.children and
                     not (game.si.game_flags & GI.GT_CHILDREN)):
@@ -670,6 +685,10 @@ class SelectGameDialogWithPreview(SelectGameDialog):
         d = SelectGameAdvancedSearch(self.top, _("Advanced search"),
                                      self.criteria)
         if d.status == 0 and d.button == 0:
+            self.criteria = SearchCriteria()
+            self.performSearch()
+
+        if d.status == 0 and d.button == 1:
             self.criteria.name = d.name.get()
 
             self.list_searchtext.delete(0, "end")
@@ -711,8 +730,47 @@ class SelectGameDialogWithPreview(SelectGameDialog):
         self.tree.n_selections += 1
         self.tree.updateSelection(game)
         self.updatePreview(game)
+        self.speakGameInfo(sel)
         self.list["cursor"] = oldcur
 
+    _resizeHandlerID = None
+
+    def _resizeHandler(self):
+        self._resizeHandlerID = None
+        self.preview.canvas.config(scrollregion=(0, 0, 0, 0))
+        self.preview_game.resizeGame()
+
+    def _configureHandler(self, event=None):
+        if False:  # if not USE_PIL:
+            return
+        if not self.app:
+            return
+        if (not self.app.opt.auto_scale and
+                not self.app.opt.preview_scale):
+            return
+        if self._resizeHandlerID:
+            self.preview.canvas.after_cancel(self._resizeHandlerID)
+        self._resizeHandlerID = self.preview.canvas.after(300,
+                                                          self._resizeHandler)
+        # should return EVENT_HANDLED or EVENT_PROPAGATE explicitly.
+
+    def speakGameInfo(self, name):
+        message = name
+        info = self.preview_game.gameinfo
+        message += " - " + _("Name:") + " " + info.name
+        if len(info.altnames) > 0:
+            message += (" - " + _("Alternate Names:") + " " +
+                        ' '.join(info.altnames))
+        message += (" - " + _("Category:") + " " +
+                    _(CSI.TYPE[info.category]))
+        if info.si.game_type in GI.TYPE_NAMES:
+            message += (" - " + _("Type:") + " " +
+                        _(GI.TYPE_NAMES[info.si.game_type]))
+        if info.skill_level is not None:
+            message += (" - " + _("Skill Level:") + " " +
+                        GI.SKILL_LEVELS.get(info.skill_level))
+        self.app.speech.speak(message)
+
     def updatePreview(self, gameid, animations=10):
         if gameid == self.preview_key:
             return
@@ -737,6 +795,7 @@ class SelectGameDialogWithPreview(SelectGameDialog):
                 menubar=None,
                 miscrandom=self.app.miscrandom,
                 opt=self.app.opt.copy(),
+                speech=self.app.speech,
                 startup_opt=self.app.startup_opt,
                 stats=self.app.stats.new(),
                 top=None,
@@ -767,9 +826,17 @@ class SelectGameDialogWithPreview(SelectGameDialog):
                 if c:
                     c2 = c.get(cardset.subtype)
         if c2:
-            self.preview_app.images = c2[2]
+            if self.app.opt.auto_scale and self.app.opt.preview_scale:
+                self.preview_app.images = c2[1]
+            else:
+                self.preview_app.images = c2[2]
         else:
-            self.preview_app.images = self.app.subsampled_images
+            if self.app.opt.auto_scale and self.app.opt.preview_scale:
+                self.preview_app.images = self.app.images
+            else:
+                self.preview_app.images = self.app.subsampled_images
+        if self.app.opt.auto_scale and self.app.opt.preview_scale:
+            self.preview_app.images.setNegative(self.app.opt.negative_bottom)
 
         self.preview_app.audio = None    # turn off audio for initial dealing
         if animations >= 0:
@@ -793,10 +860,14 @@ class SelectGameDialogWithPreview(SelectGameDialog):
             self.preview_game.restoreGameFromBookmark(self.bookmark)
         else:
             self.preview_game.newGame(random=random, autoplay=1)
-        gw, gh = self.preview_game.width, self.preview_game.height
-        canvas.config(scrollregion=(0, 0, gw, gh))
-        canvas.xview_moveto(0)
-        canvas.yview_moveto(0)
+        if (not self.app.opt.auto_scale and
+                not self.app.opt.preview_scale):
+            gw, gh = self.preview_game.width, self.preview_game.height
+            canvas.config(scrollregion=(0, 0, gw, gh))
+            canvas.xview_moveto(0)
+            canvas.yview_moveto(0)
+        else:
+            canvas.config(scrollregion=(0, 0, 0, 0))
         #
         self.preview_app.audio = self.app.audio
         if self.app.opt.animations:
@@ -945,6 +1016,10 @@ class SearchCriteria:
                                   "Games not played": "not played"}
 
 
+def _extract_names(game_info):
+    return [""] + [name for name, _games in game_info]
+
+
 class SelectGameAdvancedSearch(MfxDialog):
     def __init__(self, parent, title, criteria, **kw):
         kw = self.initKw(kw)
@@ -1004,17 +1079,17 @@ class SelectGameAdvancedSearch(MfxDialog):
         #
         row = 0
 
-        labelName = tkinter.Label(top_frame, text="Name:", anchor="w")
+        labelName = tkinter.Label(top_frame, text=_("Name:"), anchor="w")
         labelName.grid(row=row, column=0, columnspan=1, sticky='ew',
                        padx=1, pady=1)
-        textName = tkinter.Entry(top_frame, textvariable=self.name)
+        textName = PysolEntry(top_frame, textvariable=self.name,
+                              fieldname=_("Name:"))
         textName.grid(row=row, column=1, columnspan=4, sticky='ew',
                       padx=1, pady=1)
         row += 1
 
-        altCheck = tkinter.Checkbutton(top_frame, variable=self.usealt,
-                                       text=_("Check alternate names"),
-                                       anchor="w")
+        altCheck = PysolCheckbutton(top_frame, variable=self.usealt,
+                                    text=_("Check alternate names"))
         altCheck.grid(row=row, column=1, columnspan=4, sticky='ew',
                       padx=1, pady=1)
         row += 1
@@ -1024,10 +1099,12 @@ class SelectGameAdvancedSearch(MfxDialog):
 
         self.categoryValues = criteria.categoryOptions
 
-        labelCategory = tkinter.Label(top_frame, text="Category:", anchor="w")
+        labelCategory = tkinter.Label(top_frame, text=_("Category:"),
+                                      anchor="w")
         labelCategory.grid(row=row, column=0, columnspan=1, sticky='ew',
                            padx=1, pady=1)
         textCategory = PysolCombo(top_frame, values=categoryValues,
+                                  fieldname=_("Category:"),
                                   textvariable=self.category, state='readonly',
                                   selectcommand=self.updateSubcategories)
         textCategory.grid(row=row, column=1, columnspan=4, sticky='ew',
@@ -1037,11 +1114,12 @@ class SelectGameAdvancedSearch(MfxDialog):
         subcategoryValues = list(criteria.subcategoryOptions.keys())
         subcategoryValues.sort()
 
-        labelSubcategory = tkinter.Label(top_frame, text="Subcategory:",
+        labelSubcategory = tkinter.Label(top_frame, text=_("Subcategory:"),
                                          anchor="w")
         labelSubcategory.grid(row=row, column=0, columnspan=1, sticky='ew',
                               padx=1, pady=1)
         textSubcategory = PysolCombo(top_frame, values=subcategoryValues,
+                                     fieldname=_("Subcategory:"),
                                      textvariable=self.subcategory,
                                      state='readonly')
         textSubcategory.grid(row=row, column=1, columnspan=4, sticky='ew',
@@ -1053,10 +1131,11 @@ class SelectGameAdvancedSearch(MfxDialog):
         typeValues = list(criteria.typeOptions.keys())
         typeValues.sort()
 
-        labelType = tkinter.Label(top_frame, text="Type:", anchor="w")
+        labelType = tkinter.Label(top_frame, text=_("Type:"), anchor="w")
         labelType.grid(row=row, column=0, columnspan=1, sticky='ew',
                        padx=1, pady=1)
         textType = PysolCombo(top_frame, values=typeValues,
+                              fieldname=_("Type:"),
                               textvariable=self.type, state='readonly')
         textType.grid(row=row, column=1, columnspan=4, sticky='ew',
                       padx=1, pady=1)
@@ -1064,10 +1143,12 @@ class SelectGameAdvancedSearch(MfxDialog):
 
         skillValues = list(criteria.skillOptions.keys())
 
-        labelSkill = tkinter.Label(top_frame, text="Skill level:", anchor="w")
+        labelSkill = tkinter.Label(top_frame, text=_("Skill level:"),
+                                   anchor="w")
         labelSkill.grid(row=row, column=0, columnspan=1, sticky='ew',
                         padx=1, pady=1)
         textSkill = PysolCombo(top_frame, values=skillValues,
+                               fieldname=_("Skill level:"),
                                textvariable=self.skill, state='readonly')
         textSkill.grid(row=row, column=1, columnspan=4, sticky='ew',
                        padx=1, pady=1)
@@ -1075,10 +1156,11 @@ class SelectGameAdvancedSearch(MfxDialog):
 
         deckValues = list(criteria.deckOptions.keys())
 
-        labelDecks = tkinter.Label(top_frame, text="Decks:", anchor="w")
+        labelDecks = tkinter.Label(top_frame, text=_("Decks:"), anchor="w")
         labelDecks.grid(row=row, column=0, columnspan=1, sticky='ew',
                         padx=1, pady=1)
         textDecks = PysolCombo(top_frame, values=deckValues,
+                               fieldname=_("Decks:"),
                                textvariable=self.decks, state='readonly')
         textDecks.grid(row=row, column=1, columnspan=4, sticky='ew',
                        padx=1, pady=1)
@@ -1086,60 +1168,56 @@ class SelectGameAdvancedSearch(MfxDialog):
 
         redealValues = list(criteria.redealOptions.keys())
 
-        labelRedeals = tkinter.Label(top_frame, text="Redeals:", anchor="w")
+        labelRedeals = tkinter.Label(top_frame, text=_("Redeals:"), anchor="w")
         labelRedeals.grid(row=row, column=0, columnspan=1, sticky='ew',
                           padx=1, pady=1)
         textRedeals = PysolCombo(top_frame, values=redealValues,
+                                 fieldname=_("Redeals:"),
                                  textvariable=self.redeals, state='readonly')
         textRedeals.grid(row=row, column=1, columnspan=4, sticky='ew',
                          padx=1, pady=1)
         row += 1
 
-        compatValues = list()
-        compatValues.append("")
-        for name, games in GI.GAMES_BY_COMPATIBILITY:
-            compatValues.append(name)
-
-        labelCompat = tkinter.Label(top_frame, text="Compatibility:",
+        labelCompat = tkinter.Label(top_frame, text=_("Compatibility:"),
                                     anchor="w")
         labelCompat.grid(row=row, column=0, columnspan=1, sticky='ew',
                          padx=1, pady=1)
-        textCompat = PysolCombo(top_frame, values=compatValues,
+        textCompat = PysolCombo(top_frame, values=_extract_names(
+                                    GI.GAMES_BY_COMPATIBILITY),
+                                fieldname=_("Compatibility:"),
                                 textvariable=self.compat, state='readonly')
         textCompat.grid(row=row, column=1, columnspan=4, sticky='ew',
                         padx=1, pady=1)
         row += 1
 
-        inventorValues = list()
-        inventorValues.append("")
-        for name, games in GI.GAMES_BY_INVENTORS:
-            inventorValues.append(name)
-
-        labelInventor = tkinter.Label(top_frame, text="Inventor:", anchor="w")
+        labelInventor = tkinter.Label(top_frame, text=_("Inventor:"),
+                                      anchor="w")
         labelInventor.grid(row=row, column=0, columnspan=1, sticky='ew',
                            padx=1, pady=1)
-        textInventor = PysolCombo(top_frame, values=inventorValues,
+        textInventor = PysolCombo(top_frame,
+                                  values=_extract_names(GI.GAMES_BY_INVENTORS),
+                                  fieldname=_("Inventor:"),
                                   textvariable=self.inventor, state='readonly')
         textInventor.grid(row=row, column=1, columnspan=4, sticky='ew',
                           padx=1, pady=1)
         row += 1
 
         versionCompareValues = list(criteria.versionCompareOptions)
-        versionValues = list()
-        versionValues.append("")
-        for name, games in GI.GAMES_BY_PYSOL_VERSION:
-            versionValues.append(name)
 
-        labelVersion = tkinter.Label(top_frame, text="PySol Version:",
+        labelVersion = tkinter.Label(top_frame, text=_("PySol version:"),
                                      anchor="w")
         labelVersion.grid(row=row, column=0, columnspan=1, sticky='ew',
                           padx=1, pady=1)
         textVersionCompare = PysolCombo(top_frame, values=versionCompareValues,
+                                        fieldname=_("PySol version:"),
                                         textvariable=self.versioncompare,
                                         state='readonly')
         textVersionCompare.grid(row=row, column=1, columnspan=2, sticky='ew',
                                 padx=1, pady=1)
-        textVersion = PysolCombo(top_frame, values=versionValues,
+
+        textVersion = PysolCombo(top_frame, values=_extract_names(
+                                     GI.GAMES_BY_PYSOL_VERSION),
+                                 fieldname=_("PySol version:"),
                                  textvariable=self.version, state='readonly')
         textVersion.grid(row=row, column=3, columnspan=2, sticky='ew',
                          padx=1, pady=1)
@@ -1147,80 +1225,85 @@ class SelectGameAdvancedSearch(MfxDialog):
 
         statisticsValues = list(criteria.statisticsOptions.keys())
 
-        labelStats = tkinter.Label(top_frame, text="Statistics:", anchor="w")
+        labelStats = tkinter.Label(top_frame, text=_("Play history:"),
+                                   anchor="w")
         labelStats.grid(row=row, column=0, columnspan=1, sticky='ew',
                         padx=1, pady=1)
         textStats = PysolCombo(top_frame, values=statisticsValues,
+                               fieldname=_("Play history:"),
                                textvariable=self.statistics, state='readonly')
         textStats.grid(row=row, column=1, columnspan=4, sticky='ew',
                        padx=1, pady=1)
         row += 1
 
         col = 0
-        popularCheck = tkinter.Checkbutton(top_frame, variable=self.popular,
-                                           text=_("Popular"), anchor="w")
+        popularCheck = PysolCheckbutton(top_frame, variable=self.popular,
+                                        text=_("Popular"))
         popularCheck.grid(row=row, column=col, columnspan=1, sticky='ew',
                           padx=1, pady=1)
         col += 2
 
-        recentCheck = tkinter.Checkbutton(top_frame, variable=self.recent,
-                                          text=_("Recent"), anchor="w")
+        recentCheck = PysolCheckbutton(top_frame, variable=self.recent,
+                                       text=_("Recent"))
         recentCheck.grid(row=row, column=col, columnspan=1, sticky='ew',
                          padx=1, pady=1)
         col += 2
 
-        favoriteCheck = tkinter.Checkbutton(top_frame, variable=self.favorite,
-                                            text=_("Favorite"), anchor="w")
+        favoriteCheck = PysolCheckbutton(top_frame, variable=self.favorite,
+                                         text=_("Favorite"))
         favoriteCheck.grid(row=row, column=col, columnspan=1, sticky='ew',
                            padx=1, pady=1)
 
         row += 1
         col = 0
 
-        childCheck = tkinter.Checkbutton(top_frame, variable=self.children,
-                                         text=_("Children's"), anchor="w")
+        childCheck = PysolCheckbutton(top_frame, variable=self.children,
+                                      text=_("Children's"))
         childCheck.grid(row=row, column=col, columnspan=1, sticky='ew',
                         padx=1, pady=1)
         col += 2
 
-        scoreCheck = tkinter.Checkbutton(top_frame, variable=self.scoring,
-                                         text=_("Scored"), anchor="w")
+        scoreCheck = PysolCheckbutton(top_frame, variable=self.scoring,
+                                      text=_("Scored"))
         scoreCheck.grid(row=row, column=col, columnspan=1, sticky='ew',
                         padx=1, pady=1)
         col += 2
 
-        stripCheck = tkinter.Checkbutton(top_frame, variable=self.stripped,
-                                         text=_("Stripped Deck"), anchor="w")
+        stripCheck = PysolCheckbutton(top_frame, variable=self.stripped,
+                                      text=_("Stripped Deck"))
         stripCheck.grid(row=row, column=col, columnspan=1, sticky='ew',
                         padx=1, pady=1)
         row += 1
         col = 0
 
-        sepCheck = tkinter.Checkbutton(top_frame, variable=self.separate,
-                                       text=_("Separate Decks"), anchor="w")
+        sepCheck = PysolCheckbutton(top_frame, variable=self.separate,
+                                    text=_("Separate Decks"))
         sepCheck.grid(row=row, column=col, columnspan=1, sticky='ew',
                       padx=1, pady=1)
         col += 2
 
-        openCheck = tkinter.Checkbutton(top_frame, variable=self.open,
-                                        text=_("Open"), anchor="w")
+        openCheck = PysolCheckbutton(top_frame, variable=self.open,
+                                     text=_("Open"))
         openCheck.grid(row=row, column=col, columnspan=1, sticky='ew',
                        padx=1, pady=1)
         col += 2
 
-        relaxedCheck = tkinter.Checkbutton(top_frame, variable=self.relaxed,
-                                           text=_("Relaxed"), anchor="w")
+        relaxedCheck = PysolCheckbutton(top_frame, variable=self.relaxed,
+                                        text=_("Relaxed"))
         relaxedCheck.grid(row=row, column=col, columnspan=1, sticky='ew',
                           padx=1, pady=1)
         row += 1
         col = 0
 
-        originalCheck = tkinter.Checkbutton(top_frame, variable=self.original,
-                                            text=_("Original"), anchor="w")
+        originalCheck = PysolCheckbutton(top_frame, variable=self.original,
+                                         text=_("Original"))
         originalCheck.grid(row=row, column=col, columnspan=1, sticky='ew',
                            padx=1, pady=1)
 
         focus = self.createButtons(bottom_frame, kw)
+
+        top_frame.config(width=bottom_frame.winfo_width())
+
         # focus = text_w
         self.mainloop(focus, kw.timeout)
 
@@ -1243,7 +1326,7 @@ class SelectGameAdvancedSearch(MfxDialog):
 
     def initKw(self, kw):
         kw = KwStruct(kw,
-                      strings=(_("&OK"), _("&Cancel")), default=0,
-                      padx=10, pady=10,
+                      strings=(_("C&lear"), 'sep', _("&OK"), _("&Cancel")),
+                      default=1, padx=10, pady=10,
                       )
         return MfxDialog.initKw(self, kw)
