@@ -22,20 +22,20 @@
 # ---------------------------------------------------------------------------##
 
 import os
+import tkinter
+import tkinter.colorchooser
+import tkinter.ttk as ttk
 
 from pysollib.mfxutil import KwStruct, USE_PIL
 from pysollib.mygettext import _
+from pysollib.resource import TTI
 from pysollib.ui.tktile.selecttree import SelectDialogTreeData
 from pysollib.ui.tktile.tkutil import bind
 
-import six
-from six.moves import tkinter
-from six.moves import tkinter_colorchooser
-from six.moves import tkinter_ttk as ttk
-
 from .selecttree import SelectDialogTreeCanvas
 from .selecttree import SelectDialogTreeLeaf, SelectDialogTreeNode
-from .tkwidget import MfxDialog, MfxScrolledCanvas, PysolCombo
+from .tkwidget import MfxDialog, MfxScrolledCanvas, PysolButton, PysolCombo, \
+                      PysolEntry, PysolNotebook
 
 
 # ************************************************************************
@@ -84,14 +84,27 @@ class SelectTileData(SelectDialogTreeData):
                 None, _("Images"),
                 lambda tile: (os.path.basename(
                     os.path.dirname(tile.filename)) in
-                              ('stretch', 'save-aspect')), expanded=0),
+                              ('stretch', 'save-aspect', 'stretch-4k',
+                               'save-aspect-4k')), expanded=0),
             SelectTileNode(None, _("Solid Colors"), (
-                SelectTileLeaf(None, None, _("Blue"), key="#0082df"),
+                SelectTileLeaf(None, None, _("Azure"), key="#0082df"),
+                SelectTileLeaf(None, None, _("Black"), key="#000000"),
+                SelectTileLeaf(None, None, _("Blue"), key="#0000ff"),
+                SelectTileLeaf(None, None, _("Bright Green"), key="#00ff00"),
+                SelectTileLeaf(None, None, _("Brown"), key="#684700"),
+                SelectTileLeaf(None, None, _("Cyan"), key="#00ffff"),
+                SelectTileLeaf(None, None, _("Grey"), key="#888888"),
                 SelectTileLeaf(None, None, _("Green"), key="#008200"),
+                SelectTileLeaf(None, None, _("Magenta"), key="#ff00ff"),
                 SelectTileLeaf(None, None, _("Navy"), key="#000086"),
                 SelectTileLeaf(None, None, _("Olive"), key="#868200"),
                 SelectTileLeaf(None, None, _("Orange"), key="#f79600"),
+                SelectTileLeaf(None, None, _("Pink"), key="#ff92fc"),
+                SelectTileLeaf(None, None, _("Purple"), key="#8300ff"),
+                SelectTileLeaf(None, None, _("Red"), key="#ff0000"),
                 SelectTileLeaf(None, None, _("Teal"), key="#008286"),
+                SelectTileLeaf(None, None, _("White"), key="#ffffff"),
+                SelectTileLeaf(None, None, _("Yellow"), key="#ffff00"),
             ), expanded=0),
         )
 
@@ -151,7 +164,7 @@ class SelectTileDialogWithPreview(MfxDialog):
         paned_window.add(left_frame)
         paned_window.add(right_frame)
 
-        notebook = ttk.Notebook(left_frame)
+        notebook = PysolNotebook(left_frame)
         notebook.grid(row=0, column=0, sticky='nsew',
                       padx=padx, pady=pady, columnspan=2)
         tree_frame = ttk.Frame(notebook)
@@ -169,14 +182,17 @@ class SelectTileDialogWithPreview(MfxDialog):
         # Search
         searchbox = ttk.Frame(search_frame)
         searchText = tkinter.StringVar()
-        self.list_searchlabel = tkinter.Label(searchbox, text="Search:",
+        self.list_searchlabel = tkinter.Label(searchbox, text=_("Search:"),
                                               justify='left', anchor='w')
         self.list_searchlabel.pack(side="top", fill='both', ipadx=1)
-        self.list_searchtext = tkinter.Entry(searchbox,
-                                             textvariable=searchText)
+        self.list_searchtext = PysolEntry(searchbox,
+                                          fieldname=_("Search:"),
+                                          textvariable=searchText)
 
-        self.advSearch = tkinter.Button(searchbox, text='...',
-                                        command=self.advancedSearch)
+        self.advSearch = PysolButton(searchbox, text='...',
+                                     prefixtext=_("Advanced Search"),
+                                     command=self.advancedSearch,
+                                     width=2)
         self.advSearch.pack(side="right")
 
         self.list_searchtext.pack(side="top", fill='both',
@@ -194,6 +210,8 @@ class SelectTileDialogWithPreview(MfxDialog):
                        fill='both', ipadx=1)
         self.updateSearchList("")
         bind(self.list, '<<ListboxSelect>>', self.selectSearchResult)
+        bind(self.list, '<FocusIn>',
+             lambda e: self.app.speech.speak(_("Tiles list")))
         bind(self.list, '<FocusOut>',
              lambda e: self.list.selection_clear(0, 'end'))
 
@@ -211,10 +229,12 @@ class SelectTileDialogWithPreview(MfxDialog):
             self.scaling.set(next(key for key, value in
                                   self.scaleOptions.items()
                              if value == app.opt.tabletile_scale_method))
-            self.labelScale = tkinter.Label(left_frame, text="Image scaling:",
+            self.labelScale = tkinter.Label(left_frame,
+                                            text=_("Image scaling:"),
                                             anchor="w")
 
             self.textScale = PysolCombo(left_frame, values=scaleValues,
+                                        fieldname=_("Image scaling:"),
                                         state='readonly',
                                         textvariable=self.scaling,
                                         selectcommand=self.updateScaling)
@@ -252,7 +272,7 @@ class SelectTileDialogWithPreview(MfxDialog):
     def initKw(self, kw):
         kw = KwStruct(kw,
                       strings=((_("&Solid color..."), 10),
-                               'sep', _("&OK"), _("&Cancel"),),
+                               'sep', _("&Select"), _("&Cancel"),),
                       default=0,
                       resizable=True,
                       font=None,
@@ -262,14 +282,14 @@ class SelectTileDialogWithPreview(MfxDialog):
 
     def mDone(self, button):
         if button == 0:  # "OK" or double click
-            if isinstance(self.tree.selection_key, six.string_types):
+            if isinstance(self.tree.selection_key, str):
                 self.key = str(self.tree.selection_key)
             else:
                 self.key = self.tree.selection_key
             self.tree.n_expansions = 1  # save xyview in any case
         if button == 10:  # "Solid color..."
             try:
-                c = tkinter_colorchooser.askcolor(
+                c = tkinter.colorchooser.askcolor(
                     master=self.top,
                     initialcolor=self.table_color,
                     title=_("Select table color"))
@@ -308,12 +328,16 @@ class SelectTileDialogWithPreview(MfxDialog):
 
             if (self.criteria.type == "Images" and os.path.basename(
                     os.path.dirname(tile.filename)) not in
-                    ('stretch', 'save-aspect')):
+                    ('stretch', 'save-aspect', 'stretch-4k',
+                     'save-aspect-4k')):
                 continue
-
             if (self.criteria.type == "Tiles" and os.path.basename(
                     os.path.dirname(tile.filename)) != 'tiles'):
                 continue
+            if (self.criteria.size != ""
+                    and self.criteria.sizeOptions[self.criteria.size]
+                    != tile.size):
+                continue
 
             if self.app.checkSearchString(self.criteria.name,
                                           tile.name):
@@ -328,12 +352,17 @@ class SelectTileDialogWithPreview(MfxDialog):
         d = SelectTileAdvancedSearch(self.top, _("Advanced search"),
                                      self.criteria)
         if d.status == 0 and d.button == 0:
+            self.criteria = SearchCriteria()
+            self.performSearch()
+
+        if d.status == 0 and d.button == 1:
             self.criteria.name = d.name.get()
 
             self.list_searchtext.delete(0, "end")
             self.list_searchtext.insert(0, d.name.get())
 
             self.criteria.type = d.type.get()
+            self.criteria.size = d.size.get()
 
             self.performSearch()
 
@@ -348,8 +377,12 @@ class SelectTileDialogWithPreview(MfxDialog):
         self.tree.n_selections += 1
         self.tree.updateSelection(cardset)
         self.updatePreview(cardset)
+        self.speakTileInfo(sel)
         self.list["cursor"] = oldcur
 
+    def speakTileInfo(self, name):
+        self.app.speech.speak(name)
+
     def updatePreview(self, key, scaling=-1):
         if scaling < 0:
             scaling = self.preview_scaling
@@ -359,7 +392,7 @@ class SelectTileDialogWithPreview(MfxDialog):
         canvas.deleteAllItems()
 
         self.preview_scaling = scaling
-        if isinstance(key, six.string_types):
+        if isinstance(key, str):
             if USE_PIL:
                 self.textScale['state'] = 'disabled'
             # solid color
@@ -387,6 +420,13 @@ class SearchCriteria:
     def __init__(self):
         self.name = ""
         self.type = ""
+        self.size = ""
+
+        self.sizeOptions = {"": -1,
+                            "Tile": TTI.SIZE_TILE,
+                            "SD": TTI.SIZE_SD,
+                            "HD": TTI.SIZE_HD,
+                            "4K": TTI.SIZE_4K}
         self.typeOptions = ("", "Images", "Tiles")
 
 
@@ -402,27 +442,45 @@ class SelectTileAdvancedSearch(MfxDialog):
         self.name.set(criteria.name)
         self.type = tkinter.StringVar()
         self.type.set(criteria.type)
+        self.size = tkinter.StringVar()
+        self.size.set(criteria.size)
         #
         row = 0
 
-        labelName = tkinter.Label(top_frame, text="Name:", anchor="w")
+        labelName = tkinter.Label(top_frame, text=_("Name:"), anchor="w")
         labelName.grid(row=row, column=0, columnspan=1, sticky='ew',
                        padx=1, pady=1)
-        textName = tkinter.Entry(top_frame, textvariable=self.name)
+        textName = PysolEntry(top_frame, textvariable=self.name,
+                              fieldname=_("Name:"))
         textName.grid(row=row, column=1, columnspan=4, sticky='ew',
                       padx=1, pady=1)
         row += 1
 
         typeValues = list(criteria.typeOptions)
 
-        labelType = tkinter.Label(top_frame, text="Type:", anchor="w")
+        labelType = tkinter.Label(top_frame, text=_("Type:"), anchor="w")
         labelType.grid(row=row, column=0, columnspan=1, sticky='ew',
                        padx=1, pady=1)
         textType = PysolCombo(top_frame, values=typeValues,
+                              fieldname=_("Type:"),
                               textvariable=self.type, state='readonly')
         textType.grid(row=row, column=1, columnspan=4, sticky='ew',
                       padx=1, pady=1)
         row += 1
+        if USE_PIL:
+            sizeValues = list(criteria.sizeOptions.keys())
+
+            labelSize = tkinter.Label(top_frame, text=_("Size:"), anchor="w")
+            labelSize.grid(row=row, column=0, columnspan=1, sticky='ew',
+                           padx=1, pady=1)
+            textSize = PysolCombo(top_frame, values=sizeValues,
+                                  fieldname=_("Size:"),
+                                  textvariable=self.size, state='readonly')
+            textSize.grid(row=row, column=1, columnspan=4, sticky='ew',
+                          padx=1, pady=1)
+            row += 1
+
+        top_frame.columnconfigure(4, weight=1)
 
         focus = self.createButtons(bottom_frame, kw)
         # focus = text_w
@@ -430,7 +488,7 @@ class SelectTileAdvancedSearch(MfxDialog):
 
     def initKw(self, kw):
         kw = KwStruct(kw,
-                      strings=(_("&OK"), _("&Cancel")), default=0,
-                      padx=10, pady=10,
+                      strings=(_("C&lear"), 'sep', _("&OK"), _("&Cancel")),
+                      default=1, padx=10, pady=10,
                       )
         return MfxDialog.initKw(self, kw)
