@@ -3,6 +3,8 @@ import os
 import platform
 import re
 import sys
+import tkinter
+import tkinter.filedialog
 
 from pysollib.gamedb import GI
 from pysollib.hint import PySolHintLayoutImportError
@@ -17,14 +19,11 @@ from pysollib.ui.tktile.tkconst import EVENT_PROPAGATE
 from pysollib.ui.tktile.tkconst import STATUSBAR_ITEMS, TOOLBAR_BUTTONS
 from pysollib.ui.tktile.tkutil import after_idle, bind
 
-from six.moves import tkinter
-from six.moves import tkinter_tkfiledialog
-
 
 def createToolbarMenu(menubar, menu):
     tearoff = menu.cget('tearoff')
     data_dir = os.path.join(menubar.app.dataloader.dir, 'images', 'toolbar')
-    submenu = MfxMenu(menu, label=n_('Icon Style'), tearoff=tearoff)
+    submenu = MfxMenu(menu, label=n_('Icon style'), tearoff=tearoff)
     styledirs = os.listdir(data_dir)
     styledirs.sort()
     for f in styledirs:
@@ -35,7 +34,7 @@ def createToolbarMenu(menubar, menu):
               label=name,
               variable=menubar.tkopt.toolbar_style,
               value=f, command=menubar.mOptToolbarStyle)
-    submenu = MfxMenu(menu, label=n_('Icon Size'), tearoff=tearoff)
+    submenu = MfxMenu(menu, label=n_('Icon size'), tearoff=tearoff)
     submenu.add_radiobutton(label=n_("Small icons"),
                             variable=menubar.tkopt.toolbar_size, value=0,
                             command=menubar.mOptToolbarSize)
@@ -88,6 +87,20 @@ def createStatusbarMenu(menubar, menu):
 
 def createOtherGraphicsMenu(menubar, menu):
     tearoff = menu.cget('tearoff')
+    data_dir = os.path.join(menubar.app.dataloader.dir, 'images', 'buttons')
+    submenu = MfxMenu(menu, label=n_('&Button icons'), tearoff=tearoff)
+    styledirs = os.listdir(data_dir)
+    styledirs.append("none")
+    styledirs.sort()
+    for f in styledirs:
+        d = os.path.join(data_dir, f)
+        if (os.path.isdir(d) and os.path.exists(os.path.join(d))) \
+                or f == "none":
+            name = f.replace('_', ' ').capitalize()
+            submenu.add_radiobutton(
+                label=name,
+                variable=menubar.tkopt.button_icon_style,
+                value=f, command=menubar.mOptButtonIconStyle)
     data_dir = os.path.join(menubar.app.dataloader.dir, 'images', 'demo')
     submenu = MfxMenu(menu, label=n_('&Demo logo'), tearoff=tearoff)
     styledirs = os.listdir(data_dir)
@@ -266,6 +279,9 @@ class PysolMenubarTkCommon:
         self._createMenubar()
         self.top = top
 
+        # Sometimes, this needs to be tracked between methods
+        self.wasPaused = False
+
         if self.progress:
             self.progress.update(step=1)
 
@@ -278,7 +294,6 @@ class PysolMenubarTkCommon:
         self.tkopt = Struct(
             gameid=tkinter.IntVar(),
             gameid_popular=tkinter.IntVar(),
-            comment=tkinter.BooleanVar(),
             autofaceup=tkinter.BooleanVar(),
             autodrop=tkinter.BooleanVar(),
             autodeal=tkinter.BooleanVar(),
@@ -300,11 +315,13 @@ class PysolMenubarTkCommon:
             pegged_auto_remove=tkinter.BooleanVar(),
             sound=tkinter.BooleanVar(),
             auto_scale=tkinter.BooleanVar(),
+            preview_scale=tkinter.BooleanVar(),
             preserve_aspect_ratio=tkinter.BooleanVar(),
             resampling=tkinter.IntVar(),
             spread_stacks=tkinter.BooleanVar(),
             center_layout=tkinter.BooleanVar(),
             save_games_geometry=tkinter.BooleanVar(),
+            topmost_dialogs=tkinter.BooleanVar(),
             cardback=tkinter.IntVar(),
             tabletile=tkinter.IntVar(),
             animations=tkinter.IntVar(),
@@ -326,6 +343,7 @@ class PysolMenubarTkCommon:
             num_cards=tkinter.BooleanVar(),
             helpbar=tkinter.BooleanVar(),
             splashscreen=tkinter.BooleanVar(),
+            button_icon_style=tkinter.StringVar(),
             demo_logo=tkinter.BooleanVar(),
             demo_logo_style=tkinter.StringVar(),
             pause_text_style=tkinter.StringVar(),
@@ -334,6 +352,7 @@ class PysolMenubarTkCommon:
             tree_icon_style=tkinter.StringVar(),
             mouse_type=tkinter.StringVar(),
             mouse_undo=tkinter.BooleanVar(),
+            mouse_dragcursor=tkinter.BooleanVar(),
             negative_bottom=tkinter.BooleanVar(),
             pause=tkinter.BooleanVar(),
             theme=tkinter.StringVar(),
@@ -373,11 +392,13 @@ class PysolMenubarTkCommon:
         tkopt.pegged_auto_remove.set(opt.pegged_auto_remove)
         tkopt.sound.set(opt.sound)
         tkopt.auto_scale.set(opt.auto_scale)
+        tkopt.preview_scale.set(opt.preview_scale)
         tkopt.preserve_aspect_ratio.set(opt.preserve_aspect_ratio)
         tkopt.resampling.set(opt.resampling)
         tkopt.spread_stacks.set(opt.spread_stacks)
         tkopt.center_layout.set(opt.center_layout)
         tkopt.save_games_geometry.set(opt.save_games_geometry)
+        tkopt.topmost_dialogs.set(opt.topmost_dialogs)
         tkopt.cardback.set(self.app.cardset.backindex)
         tkopt.tabletile.set(self.app.tabletile_index)
         tkopt.animations.set(opt.animations)
@@ -395,6 +416,7 @@ class PysolMenubarTkCommon:
         tkopt.statusbar.set(opt.statusbar)
         # tkopt.num_cards.set(opt.num_cards)
         # tkopt.helpbar.set(opt.helpbar)
+        tkopt.button_icon_style.set(opt.button_icon_style)
         tkopt.demo_logo.set(opt.demo_logo)
         if opt.demo_logo:
             tkopt.demo_logo_style.set(opt.demo_logo_style)
@@ -407,6 +429,7 @@ class PysolMenubarTkCommon:
         tkopt.splashscreen.set(opt.splashscreen)
         tkopt.mouse_type.set(opt.mouse_type)
         tkopt.mouse_undo.set(opt.mouse_undo)
+        tkopt.mouse_dragcursor.set(opt.dragcursor)
         tkopt.negative_bottom.set(opt.negative_bottom)
         for w in TOOLBAR_BUTTONS:
             tkopt.toolbar_vars[w].set(opt.toolbar_vars.get(w, False))
@@ -421,7 +444,6 @@ class PysolMenubarTkCommon:
         tkopt = self.tkopt
         tkopt.gameid.set(game.id)
         tkopt.gameid_popular.set(game.id)
-        tkopt.comment.set(bool(game.gsaveinfo.comment))
         tkopt.pause.set(self.game.pause)
         if game.canFindCard():
             self._connect_game_find_card_dialog(game)
@@ -597,15 +619,19 @@ class PysolMenubarTkCommon:
         menu.add_command(
             label=n_("&Statistics..."),
             command=self.mPlayerStats, accelerator=m+"T")
-        menu.add_command(
-            label=n_("D&emo statistics..."),
-            command=lambda: self.mPlayerStats(mode=1101))
         menu.add_command(
             label=n_("Log..."),
             command=lambda: self.mPlayerStats(mode=103))
         menu.add_separator()
-        menu.add_checkbutton(
-            label=n_("&Comments..."), variable=self.tkopt.comment,
+        menu.add_command(
+            label=n_("D&emo statistics..."),
+            command=lambda: self.mPlayerStats(mode=1101))
+        menu.add_command(
+            label=n_("Demo log..."),
+            command=lambda: self.mPlayerStats(mode=1103))
+        menu.add_separator()
+        menu.add_command(
+            label=n_("&Comments..."),
             command=self.mEditGameComment)
 
         menu = MfxMenu(self.menubar, label=n_("&Assist"))
@@ -646,7 +672,7 @@ class PysolMenubarTkCommon:
             command=self.mOptPlayerOptions, accelerator=m+'P')
         submenu = MfxMenu(menu, label=n_("&Automatic play"))
         submenu.add_checkbutton(
-            label=n_("Auto &face up"), variable=self.tkopt.autofaceup,
+            label=n_("Auto &face-up"), variable=self.tkopt.autofaceup,
             command=self.mOptAutoFaceUp)
         submenu.add_checkbutton(
             label=n_("A&uto drop"), variable=self.tkopt.autodrop,
@@ -736,12 +762,16 @@ class PysolMenubarTkCommon:
                 label=n_("&Auto scaling"), variable=self.tkopt.auto_scale,
                 command=self.mOptAutoScale, accelerator=m+'0')
             submenu.add_checkbutton(
-                label=n_("&Preserve aspect ratio"),
+                label=n_("&Preview scaling"),
+                variable=self.tkopt.preview_scale,
+                command=self.mOptPreviewScale)
+            submenu.add_checkbutton(
+                label=n_("Pr&eserve aspect ratio"),
                 variable=self.tkopt.preserve_aspect_ratio,
                 command=self.mOptPreserveAspectRatio)
             submenu.add_separator()
             createResamplingMenu(self, submenu)
-            submenu = MfxMenu(menu, label=n_("Card la&yout"))
+            submenu = MfxMenu(menu, label=n_("Game la&yout"))
             submenu.add_checkbutton(
                 label=n_("&Spread stacks"), variable=self.tkopt.spread_stacks,
                 command=self.mOptSpreadStacks)
@@ -752,6 +782,10 @@ class PysolMenubarTkCommon:
                 label=n_("Save games &geometry"),
                 variable=self.tkopt.save_games_geometry,
                 command=self.mOptSaveGamesGeometry)
+            submenu.add_checkbutton(
+                label=n_("&Keep dialogs on top"),
+                variable=self.tkopt.topmost_dialogs,
+                command=self.mOptTopmostDialogs)
         # manager = self.app.cardset_manager
         # n = manager.len()
         menu.add_command(
@@ -836,6 +870,10 @@ class PysolMenubarTkCommon:
             value='sticky-mouse',
             command=self.mOptMouseType)
         submenu.add_separator()
+        submenu.add_checkbutton(
+            label=n_("D&rag cards cursor"),
+            variable=self.tkopt.mouse_dragcursor,
+            command=self.mOptMouseDragCursor)
         submenu.add_checkbutton(
             label=n_("Use mouse for undo/redo"),
             variable=self.tkopt.mouse_undo,
@@ -858,6 +896,10 @@ class PysolMenubarTkCommon:
                 label=n_("Save games &geometry"),
                 variable=self.tkopt.save_games_geometry,
                 command=self.mOptSaveGamesGeometry)
+            submenu.add_checkbutton(
+                label=n_("&Keep dialogs on top"),
+                variable=self.tkopt.topmost_dialogs,
+                command=self.mOptTopmostDialogs)
 
         # menu.add_checkbutton(
         #     label=n_("Startup splash sc&reen"),
@@ -890,7 +932,7 @@ class PysolMenubarTkCommon:
             label=n_("What's &new?"),
             command=self.mHelpNews)
         menu.add_command(
-            label=n_("R&eport a Bug"),
+            label=n_("R&eport a bug"),
             command=self.mHelpReportBug)
         menu.add_command(
             label=n_("&License terms"),
@@ -971,6 +1013,31 @@ class PysolMenubarTkCommon:
                 ctrl, str(i+1),
                 lambda e, i=i: self.mGotoBookmark(i, confirm=0))
 
+        # keyboard play
+        self._bindKey("", "Up", lambda e: self.mKeyboardSelect(dir=0))
+        self._bindKey("", "Down", lambda e: self.mKeyboardSelect(dir=1))
+        self._bindKey("", "Left", lambda e: self.mKeyboardSelect(dir=2))
+        self._bindKey("", "Right", lambda e: self.mKeyboardSelect(dir=3))
+
+        self._bindKey("Shift-", "Up",
+                      lambda e:self.mKeyboardSelectLayer(dir=1))
+        self._bindKey("Shift-", "Down",
+                      lambda e: self.mKeyboardSelectLayer(dir=-1))
+
+        self._bindKey("", "Tab", self.mKeyboardSelectNextType)
+        self._bindKey("Shift-", "Tab", self.mKeyboardSelectPrevType)
+
+        self._bindKey("", "plus", self.mKeyboardSelectMore)
+        self._bindKey("", "equal", self.mKeyboardSelectMore)
+        self._bindKey("", "minus", self.mKeyboardSelectLess)
+
+        self._bindKey("", "q", self.mKeyboardGameInfo)
+        self._bindKey("", "v", self.mKeyboardStackInfo)
+        self._bindKey("Shift-", "v", self.mKeyboardCoordinates)
+
+        self._bindKey("", "Return", self.mKeyboardAction)
+        self._bindKey("Shift-", "Return", self.mKeyboardAction2)
+
         # undocumented, devel
         self._bindKey(ctrl, "End", self.mPlayNextMusic)
         self._bindKey(ctrl, "Prior", self.mSelectPrevGameByName)
@@ -978,6 +1045,9 @@ class PysolMenubarTkCommon:
         self._bindKey(ctrl, "Up", self.mSelectPrevGameById)
         self._bindKey(ctrl, "Down", self.mSelectNextGameById)
 
+        self._bindKey("", "F5", self.refresh)
+        self._bindKey("", "F11", self.togglefullscreen)
+
         if os.name == 'posix' and platform.system() != 'Darwin':
             self._bindKey('Alt-', 'F4', self.mQuit)
 
@@ -1095,8 +1165,8 @@ class PysolMenubarTkCommon:
             if not mahjongg_games[n:n + d]:
                 break
             m = min(n + d - 1, len(mahjongg_games) - 1)
-            label = mahjongg_games[n].short_name[:3] + ' - ' + \
-                mahjongg_games[m].short_name[:3]
+            label = mahjongg_games[n].short_name[:3].strip() + ' - ' + \
+                mahjongg_games[m].short_name[:3].strip()
             submenu = MfxMenu(menu, label=label, name=None)
             self._addSelectGameSubSubMenu(mahjongg_games[n:n + d], submenu,
                                           command, variable, short_name=True)
@@ -1158,7 +1228,8 @@ class PysolMenubarTkCommon:
             if not games[n:n+d]:
                 break
             m = min(n+d-1, len(games)-1)
-            label = games[n].name[:3] + ' - ' + games[m].name[:3]
+            label = games[n].name[:3].strip() + ' - ' + \
+                games[m].name[:3].strip()
             submenu = MfxMenu(menu, label=label, name=None)
             self._addSelectGameSubSubMenu(games[n:n+d], submenu,
                                           command, variable)
@@ -1171,8 +1242,7 @@ class PysolMenubarTkCommon:
         # cb = (25, self.cb_max) [ len(g) > 4 * 25 ]
         # cb = min(cb, self.cb_max)
         cb = self.cb_max
-        for i in range(len(games)):
-            gi = games[i]
+        for i, gi in enumerate(games):
             columnbreak = i > 0 and (i % cb) == 0
             if short_name:
                 label = gi.short_name
@@ -1214,6 +1284,19 @@ class PysolMenubarTkCommon:
         self.tkopt.gameid_popular.set(self.game.id)
 
     def _mSelectGameDialog(self, d):
+        if self.game.pause:
+            if self.wasPaused:
+                # Nasty hack here.  This is the only way I was able to
+                # make the flow work while reliably avoiding crashes and
+                # graphical glitches when both the preview auto-scaling
+                # and the pause are in effect.
+                try:
+                    self.game.doPause()
+                except Exception:
+                    self.game.resizeGame()
+                    self.game.doPause()
+                    if self.game.pause:
+                        self.game.doPause()
         if d.status == 0 and d.button == 0 and d.gameid != self.game.id:
             self.tkopt.gameid.set(d.gameid)
             self.tkopt.gameid_popular.set(d.gameid)
@@ -1250,6 +1333,10 @@ class PysolMenubarTkCommon:
                 bookmark = self.game.gsaveinfo.bookmarks[-2][0]
                 del self.game.gsaveinfo.bookmarks[-2]
         after_idle(self.top, self.__restoreCursor)
+        self.wasPaused = False
+        if not self.game.pause:
+            self.game.doPause()
+            self.wasPaused = True
         d = self._calcSelectGameDialogWithPreview()(
             self.top, title=_("Select game"),
             app=self.app, gameid=self.game.id,
@@ -1349,10 +1436,10 @@ class PysolMenubarTkCommon:
         # insert new cardbacks
         mbacks = self.app.images.getCardbacks()
         cb = int(math.ceil(math.sqrt(len(mbacks))))
-        for i in range(len(mbacks)):
+        for i, mback in enumerate(mbacks):
             columnbreak = i > 0 and (i % cb) == 0
             submenu.add_radiobutton(
-                label=mbacks[i].name, image=mbacks[i].menu_image,
+                label=mback.name, image=mback.menu_image,
                 variable=self.tkopt.cardback, value=i,
                 command=self.mOptCardback, columnbreak=columnbreak,
                 indicatoron=0, hidemargin=0)
@@ -1375,9 +1462,6 @@ class PysolMenubarTkCommon:
         w = getattr(self.app.toolbar, path + "_button")
         w["state"] = s
 
-    def _setCommentMenu(self, v):
-        self.tkopt.comment.set(v)
-
     def _setPauseMenu(self, v):
         self.tkopt.pause.set(v)
 
@@ -1422,7 +1506,7 @@ class PysolMenubarTkCommon:
             idir, ifile = "", ""
         if not idir:
             idir = self.app.dn.savegames
-        d = tkinter_tkfiledialog.Open()
+        d = tkinter.filedialog.Open()
         filename = d.show(filetypes=self.FILETYPES,
                           defaultextension=self.DEFAULTEXTENSION,
                           initialdir=idir, initialfile=ifile)
@@ -1459,7 +1543,7 @@ Unsupported game for export.
         if not idir:
             idir = self.app.dn.boards
         # print self.game.filename, ifile
-        d = tkinter_tkfiledialog.SaveAs()
+        d = tkinter.filedialog.SaveAs()
         filename = d.show(filetypes=self.FILETYPES,
                           defaultextension=self.DEFAULTEXTENSION,
                           initialdir=idir, initialfile=ifile)
@@ -1491,7 +1575,7 @@ Unsupported game for import.
             idir, ifile = "", ""
         if not idir:
             idir = self.app.dn.boards
-        d = tkinter_tkfiledialog.Open()
+        d = tkinter.filedialog.Open()
         key = 'PYSOL_DEBUG_IMPORT'
         if key not in os.environ:
             filename = d.show(filetypes=self.FILETYPES,
@@ -1538,7 +1622,7 @@ Unsupported game for import.
         if not idir:
             idir = self.app.dn.savegames
         # print self.game.filename, ifile
-        d = tkinter_tkfiledialog.SaveAs()
+        d = tkinter.filedialog.SaveAs()
         filename = d.show(filetypes=self.FILETYPES,
                           defaultextension=self.DEFAULTEXTENSION,
                           initialdir=idir, initialfile=ifile)
@@ -1560,8 +1644,15 @@ Unsupported game for import.
     def mOptSoundDialog(self, *args):
         if self._cancelDrag(break_pause=False):
             return
+        wasPaused = False
+        if not self.game.pause:
+            self.game.doPause()
+            wasPaused = True
         self._calcSoundOptionsDialog()(
             self.top, _("Sound settings"), self.app)
+        if self.game.pause:
+            if wasPaused:
+                self.game.doPause()
         self.tkopt.sound.set(self.app.opt.sound)
 
     def mOptAutoFaceUp(self, *args):
@@ -1810,6 +1901,14 @@ Unsupported game for import.
         self.tkopt.auto_scale.set(auto_scale)
         self._updateCardSize()
 
+    def mOptPreviewScale(self, *event):
+        if self._cancelDrag(break_pause=True):
+            return
+        preview_scale = not self.app.opt.preview_scale
+
+        self.app.opt.preview_scale = preview_scale
+        self.tkopt.preview_scale.set(preview_scale)
+
     def mOptPreserveAspectRatio(self, *event):
         if self._cancelDrag(break_pause=True):
             return
@@ -1859,6 +1958,12 @@ Unsupported game for import.
             return
         self.app.opt.save_games_geometry = self.tkopt.save_games_geometry.get()
 
+    def mOptTopmostDialogs(self, *event):
+        if self._cancelDrag(break_pause=False):
+            return
+        self.app.opt.topmost_dialogs = self.tkopt.topmost_dialogs.get()
+        self.app.raiseAll()
+
     def _mOptCardback(self, index):
         if self._cancelDrag(break_pause=False):
             return
@@ -1895,11 +2000,18 @@ Unsupported game for import.
         key = self.app.tabletile_index
         if key <= 0:
             key = self.app.opt.colors['table']  # .lower()
+        wasPaused = False
+        if not self.game.pause:
+            self.game.doPause()
+            wasPaused = True
         d = self._calcSelectTileDialogWithPreview()(
             self.top, app=self.app,
             title=_("Select table background"),
             manager=self.app.tabletile_manager,
             key=key)
+        if self.game.pause:
+            if wasPaused:
+                self.game.doPause()
         if d.status == 0 and d.button == 0:
             if isinstance(d.key, str):
                 tile = self.app.tabletile_manager.get(0)
@@ -1946,6 +2058,9 @@ Unsupported game for import.
     def mOptStatusbarConfig(self, w):
         self.statusbarConfig(w, self.tkopt.statusbar_vars[w].get())
 
+    def mOptButtonIconStyle(self, *event):
+        self.setButtonIconStyle(self.tkopt.button_icon_style.get())
+
     def mOptDemoLogoStyle(self, *event):
         self.setDemoLogoStyle(self.tkopt.demo_logo_style.get())
 
@@ -1992,6 +2107,11 @@ Unsupported game for import.
             return
         self.app.opt.mouse_type = self.tkopt.mouse_type.get()
 
+    def mOptMouseDragCursor(self, *event):
+        if self._cancelDrag(break_pause=False):
+            return
+        self.app.opt.dragcursor = self.tkopt.mouse_dragcursor.get()
+
     def mOptMouseUndo(self, *event):
         if self._cancelDrag(break_pause=False):
             return
@@ -2005,6 +2125,14 @@ Unsupported game for import.
         self.game.endGame(bookmark=1)
         self.game.quitGame(bookmark=1)
 
+    def refresh(self, *event):
+        self.app.updateCardset()
+        self.game.endGame(bookmark=1)
+        self.game.quitGame(bookmark=1)
+
+    def togglefullscreen(self, *event):
+        self.app.wm_toggle_fullscreen()
+
     #
     # toolbar support
     #
@@ -2048,6 +2176,14 @@ Unsupported game for import.
             self.game.updateStatus(player=self.app.opt.player)
             self.top.update_idletasks()
 
+    def setButtonIconStyle(self, style):
+        if self._cancelDrag(break_pause=False):
+            return
+        self.app.opt.button_icon_style = style
+        self.tkopt.button_icon_style.set(style)      # update radiobutton
+        self.app.loadImages1()
+        self.app.loadImages4()
+
     def setDemoLogoStyle(self, style):
         if self._cancelDrag(break_pause=False):
             return
@@ -2105,6 +2241,10 @@ Unsupported game for import.
         self.game.quitGame(bookmark=1)
 
     def wizardDialog(self, edit=False):
+        wasPaused = False
+        if not self.game.pause:
+            self.game.doPause()
+            wasPaused = True
         from pysollib.wizardutil import write_game, reset_wizard
         WizardDialog = self._calcWizardDialog()
 
@@ -2113,6 +2253,9 @@ Unsupported game for import.
         else:
             reset_wizard(None)
         d = WizardDialog(self.top, _('Solitaire Wizard'), self.app)
+        if self.game.pause:
+            if wasPaused:
+                self.game.doPause()
         if d.status == 0 and d.button == 0:
             try:
                 if edit:
@@ -2156,7 +2299,7 @@ Error while saving game.
                                     % self.game.gameinfo.name):
             return
         from pysollib.wizardutil import delete_game
-        delete_game(self.game)
+        delete_game(self.app, self.game)
         self.game.endGame()
         self.game.quitGame(2)
 
